@@ -838,13 +838,7 @@ void BookmarksModel::removeBookmarkUrl(Bookmark *bookmark)
 
 	switch (bookmark->getType())
 	{
-		case FolderBookmark:
-			for (int i = 0; i < bookmark->rowCount(); ++i)
-			{
-				removeBookmarkUrl(bookmark->getChild(i));
-			}
-
-			break;
+		case FeedBookmark:
 		case UrlBookmark:
 			{
 				const QUrl url(Utils::normalizeUrl(bookmark->data(UrlRole).toUrl()));
@@ -860,6 +854,13 @@ void BookmarksModel::removeBookmarkUrl(Bookmark *bookmark)
 				}
 			}
 
+			break;
+		case FolderBookmark:
+			for (int i = 0; i < bookmark->rowCount(); ++i)
+			{
+				removeBookmarkUrl(bookmark->getChild(i));
+			}
+
 			break;
 		default:
 			break;
@@ -1042,7 +1043,7 @@ BookmarksModel::Bookmark* BookmarksModel::addBookmark(BookmarkType type, const Q
 
 	parent->insertRow(((index < 0) ? parent->rowCount() : index), bookmark);
 
-	if (type == UrlBookmark || type == SeparatorBookmark)
+	if (type == FeedBookmark || type == UrlBookmark || type == SeparatorBookmark)
 	{
 		bookmark->setDropEnabled(false);
 	}
