@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2015 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,11 +9,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -26,7 +26,7 @@
 namespace Otter
 {
 
-class CookieJar : public QNetworkCookieJar
+class CookieJar final : public QNetworkCookieJar
 {
 	Q_OBJECT
 
@@ -34,35 +34,35 @@ public:
 	enum CookiesPolicy
 	{
 		IgnoreCookies = 0,
-		ReadOnlyCookies = 1,
-		AcceptAllCookies = 2,
-		AcceptExistingCookies = 3
+		ReadOnlyCookies,
+		AcceptAllCookies,
+		AcceptExistingCookies
 	};
 
 	enum KeepMode
 	{
 		KeepUntilExpiresMode = 0,
-		KeepUntilExitMode = 1,
-		AskIfKeepMode = 2
+		KeepUntilExitMode,
+		AskIfKeepMode
 	};
 
 	enum CookieOperation
 	{
 		InsertCookie = 0,
-		UpdateCookie = 1,
-		RemoveCookie = 2
+		UpdateCookie,
+		RemoveCookie
 	};
 
 	explicit CookieJar(bool isPrivate, QObject *parent = nullptr);
 
 	void clearCookies(int period = 0);
-	CookieJar* clone(QObject *parent = nullptr);
-	QList<QNetworkCookie> cookiesForUrl(const QUrl &url) const;
+	CookieJar* clone(QObject *parent = nullptr) const;
+	QList<QNetworkCookie> cookiesForUrl(const QUrl &url) const override;
 	QList<QNetworkCookie> getCookiesForUrl(const QUrl &url) const;
-	QList<QNetworkCookie> getCookies(const QString &domain = QString()) const;
-	bool insertCookie(const QNetworkCookie &cookie);
-	bool updateCookie(const QNetworkCookie &cookie);
-	bool deleteCookie(const QNetworkCookie &cookie);
+	QVector<QNetworkCookie> getCookies(const QString &domain = {}) const;
+	bool insertCookie(const QNetworkCookie &cookie) override;
+	bool updateCookie(const QNetworkCookie &cookie) override;
+	bool deleteCookie(const QNetworkCookie &cookie) override;
 	bool forceInsertCookie(const QNetworkCookie &cookie);
 	bool forceUpdateCookie(const QNetworkCookie &cookie);
 	bool forceDeleteCookie(const QNetworkCookie &cookie);
@@ -70,12 +70,12 @@ public:
 	static bool isDomainTheSame(const QUrl &first, const QUrl &second);
 
 protected:
-	void timerEvent(QTimerEvent *event);
+	void timerEvent(QTimerEvent *event) override;
 	void scheduleSave();
 	void save();
 
 protected slots:
-	void optionChanged(int identifier, const QVariant &value);
+	void handleOptionChanged(int identifier, const QVariant &value);
 
 private:
 	CookiesPolicy m_generalCookiesPolicy;
