@@ -1,6 +1,7 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2015 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2017 Jan Bajer aka bajasoft <jbajer@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,33 +10,33 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
 #include "HistoryModel.h"
 #include "Console.h"
+#include "JsonSettings.h"
 #include "SessionsManager.h"
+#include "ThemesManager.h"
 #include "Utils.h"
 
 #include <QtCore/QFile>
 #include <QtCore/QJsonArray>
-#include <QtCore/QJsonDocument>
 #include <QtCore/QJsonObject>
-#include <QtCore/QSaveFile>
 
 namespace Otter
 {
 
-HistoryEntryItem::HistoryEntryItem() : QStandardItem()
+HistoryModel::Entry::Entry() : QStandardItem()
 {
 }
 
-void HistoryEntryItem::setData(const QVariant &value, int role)
+void HistoryModel::Entry::setData(const QVariant &value, int role)
 {
 	if (model() && qobject_cast<HistoryModel*>(model()))
 	{
@@ -47,12 +48,40 @@ void HistoryEntryItem::setData(const QVariant &value, int role)
 	}
 }
 
-void HistoryEntryItem::setItemData(const QVariant &value, int role)
+void HistoryModel::Entry::setItemData(const QVariant &value, int role)
 {
 	QStandardItem::setData(value, role);
 }
 
-HistoryModel::HistoryModel(const QString &path, QObject *parent) : QStandardItemModel(parent)
+QString HistoryModel::Entry::getTitle() const
+{
+	return (data(TitleRole).isNull() ? QCoreApplication::translate("Otter::HistoryEntryItem", "(Untitled)") : data(TitleRole).toString());
+}
+
+QUrl HistoryModel::Entry::getUrl() const
+{
+	return data(UrlRole).toUrl();
+}
+
+QDateTime HistoryModel::Entry::getTimeVisited() const
+{
+	return data(TimeVisitedRole).toDateTime();
+}
+
+QIcon HistoryModel::Entry::getIcon() const
+{
+	const QVariant iconData(data(Qt::DecorationRole));
+
+	return (iconData.isNull() ? ThemesManager::createIcon(QLatin1String("text-html")) : iconData.value<QIcon>());
+}
+
+quint64 HistoryModel::Entry::getIdentifier() const
+{
+	return data(IdentifierRole).toULongLong();
+}
+
+HistoryModel::HistoryModel(const QString &path, HistoryType type, QObject *parent) : QStandardItemModel(parent),
+	m_type(type)
 {
 	QFile file(path);
 
@@ -63,15 +92,17 @@ HistoryModel::HistoryModel(const QString &path, QObject *parent) : QStandardItem
 		return;
 	}
 
-	const QJsonArray array(QJsonDocument::fromJson(file.readAll()).array());
+	const QJsonArray historyArray(QJsonDocument::fromJson(file.readAll()).array());
 
 	file.close();
 
-	for (int i = 0; i < array.count(); ++i)
+	for (int i = 0; i < historyArray.count(); ++i)
 	{
-		const QJsonObject object(array.at(i).toObject());
+		const QJsonObject entryObject(historyArray.at(i).toObject());
+		QDateTime dateTime(QDateTime::fromString(entryObject.value(QLatin1String("time")).toString(), Qt::ISODate));
+		dateTime.setTimeSpec(Qt::UTC);
 
-		addEntry(QUrl(object.value(QLatin1String("url")).toString()), object.value(QLatin1String("title")).toString(), QIcon(), QDateTime::fromString(object.value(QLatin1String("time")).toString(), QLatin1String("yyyy-MM-dd hh:mm:ss")));
+		addEntry(QUrl(entryObject.value(QLatin1String("url")).toString()), entryObject.value(QLatin1String("title")).toString(), {}, dateTime);
 	}
 
 	setSortRole(TimeVisitedRole);
@@ -84,7 +115,7 @@ void HistoryModel::clearExcessEntries(int limit)
 	{
 		for (int i = (rowCount() - 1); i >= limit; --i)
 		{
-			removeEntry(index(i, 0).data(HistoryModel::IdentifierRole).toULongLong());
+			removeEntry(index(i, 0).data(IdentifierRole).toULongLong());
 		}
 	}
 }
@@ -95,6 +126,9 @@ void HistoryModel::clearRecentEntries(uint period)
 	{
 		clear();
 
+		m_urls.clear();
+		m_identifiers.clear();
+
 		emit cleared();
 
 		return;
@@ -102,7 +136,7 @@ void HistoryModel::clearRecentEntries(uint period)
 
 	for (int i = (rowCount() - 1); i >= 0; --i)
 	{
-		if (index(i, 0).data(TimeVisitedRole).toDateTime().secsTo(QDateTime::currentDateTime()) < (period * 3600))
+		if (index(i, 0).data(TimeVisitedRole).toDateTime().secsTo(QDateTime::currentDateTimeUtc()) < (period * 3600))
 		{
 			removeEntry(index(i, 0).data(IdentifierRole).toULongLong());
 		}
@@ -116,7 +150,7 @@ void HistoryModel::clearOldestEntries(int period)
 		return;
 	}
 
-	const QDateTime currentDateTime(QDateTime::currentDateTime());
+	const QDateTime currentDateTime(QDateTime::currentDateTimeUtc());
 
 	for (int i = (rowCount() - 1); i >= 0; --i)
 	{
@@ -129,14 +163,14 @@ void HistoryModel::clearOldestEntries(int period)
 
 void HistoryModel::removeEntry(quint64 identifier)
 {
-	HistoryEntryItem *entry(getEntry(identifier));
+	Entry *entry(getEntry(identifier));
 
 	if (!entry)
 	{
 		return;
 	}
 
-	const QUrl url(Utils::normalizeUrl(entry->data(UrlRole).toUrl()));
+	const QUrl url(Utils::normalizeUrl(entry->getUrl()));
 
 	if (m_urls.contains(url))
 	{
@@ -160,11 +194,24 @@ void HistoryModel::removeEntry(quint64 identifier)
 	emit modelModified();
 }
 
-HistoryEntryItem* HistoryModel::addEntry(const QUrl &url, const QString &title, const QIcon &icon, const QDateTime &date, quint64 identifier)
+HistoryModel::Entry* HistoryModel::addEntry(const QUrl &url, const QString &title, const QIcon &icon, const QDateTime &date, quint64 identifier)
 {
 	blockSignals(true);
 
-	HistoryEntryItem *entry(new HistoryEntryItem());
+	if (m_type == TypedHistory)
+	{
+		const QUrl normalizedUrl(Utils::normalizeUrl(url));
+
+		if (hasEntry(normalizedUrl))
+		{
+			for (int i = 0; i < m_urls[normalizedUrl].count(); ++i)
+			{
+				removeEntry(m_urls[normalizedUrl].at(i)->getIdentifier());
+			}
+		}
+	}
+
+	Entry *entry(new Entry());
 	entry->setIcon(icon);
 
 	insertRow(0, entry);
@@ -188,7 +235,7 @@ HistoryEntryItem* HistoryModel::addEntry(const QUrl &url, const QString &title,
 	return entry;
 }
 
-HistoryEntryItem* HistoryModel::getEntry(quint64 identifier) const
+HistoryModel::Entry* HistoryModel::getEntry(quint64 identifier) const
 {
 	if (m_identifiers.contains(identifier))
 	{
@@ -198,13 +245,13 @@ HistoryEntryItem* HistoryModel::getEntry(quint64 identifier) const
 	return nullptr;
 }
 
-QList<HistoryModel::HistoryEntryMatch> HistoryModel::findEntries(const QString &prefix, bool markAsTypedIn) const
+QVector<HistoryModel::HistoryEntryMatch> HistoryModel::findEntries(const QString &prefix, bool markAsTypedIn) const
 {
-	QList<HistoryEntryItem*> matchedEntries;
-	QList<HistoryModel::HistoryEntryMatch> allMatches;
-	QList<HistoryModel::HistoryEntryMatch> currentMatches;
-	QMultiMap<QDateTime, HistoryModel::HistoryEntryMatch> matchesMap;
-	QHash<QUrl, QList<HistoryEntryItem*> >::const_iterator urlsIterator;
+	QVector<Entry*> matchedEntries;
+	QVector<HistoryEntryMatch> allMatches;
+	QVector<HistoryEntryMatch> currentMatches;
+	QMultiMap<QDateTime, HistoryEntryMatch> matchesMap;
+	QHash<QUrl, QVector<Entry*> >::const_iterator urlsIterator;
 
 	for (urlsIterator = m_urls.constBegin(); urlsIterator != m_urls.constEnd(); ++urlsIterator)
 	{
@@ -232,7 +279,7 @@ QList<HistoryModel::HistoryEntryMatch> HistoryModel::findEntries(const QString &
 		}
 	}
 
-	currentMatches = matchesMap.values();
+	currentMatches = matchesMap.values().toVector();
 
 	matchesMap.clear();
 
@@ -244,6 +291,11 @@ QList<HistoryModel::HistoryEntryMatch> HistoryModel::findEntries(const QString &
 	return allMatches;
 }
 
+HistoryModel::HistoryType HistoryModel::getType() const
+{
+	return m_type;
+}
+
 bool HistoryModel::save(const QString &path) const
 {
 	if (SessionsManager::isReadOnly())
@@ -251,41 +303,27 @@ bool HistoryModel::save(const QString &path) const
 		return false;
 	}
 
-	QSaveFile file(path);
-
-	if (!file.open(QIODevice::WriteOnly))
-	{
-		return false;
-	}
-
-	QJsonArray array;
+	QJsonArray historyArray;
 
 	for (int i = 0; i < rowCount(); ++i)
 	{
-		QStandardItem *entry(item(i));
+		const QModelIndex index(this->index(i, 0));
 
-		if (entry)
+		if (index.isValid())
 		{
-			QJsonObject object;
-			object.insert(QLatin1String("url"), entry->data(UrlRole).toUrl().toString());
-			object.insert(QLatin1String("title"), entry->data(TitleRole).toString());
-			object.insert(QLatin1String("time"), entry->data(TimeVisitedRole).toDateTime().toString(QLatin1String("yyyy-MM-dd hh:mm:ss")));
-
-			array.prepend(object);
+			historyArray.prepend(QJsonObject({{QLatin1String("url"), index.data(UrlRole).toUrl().toString()}, {QLatin1String("title"), index.data(TitleRole).toString()}, {QLatin1String("time"), index.data(TimeVisitedRole).toDateTime().toString(Qt::ISODate)}}));
 		}
 	}
 
-	QJsonDocument document;
-	document.setArray(array);
+	JsonSettings settings;
+	settings.setArray(historyArray);
 
-	file.write(document.toJson(QJsonDocument::Indented));
-
-	return file.commit();
+	return settings.save(path);
 }
 
 bool HistoryModel::setData(const QModelIndex &index, const QVariant &value, int role)
 {
-	HistoryEntryItem *entry(dynamic_cast<HistoryEntryItem*>(itemFromIndex(index)));
+	Entry *entry(static_cast<Entry*>(itemFromIndex(index)));
 
 	if (!entry)
 	{
@@ -311,7 +349,7 @@ bool HistoryModel::setData(const QModelIndex &index, const QVariant &value, int
 		{
 			if (!m_urls.contains(newUrl))
 			{
-				m_urls[newUrl] = QList<HistoryEntryItem*>();
+				m_urls[newUrl] = QVector<Entry*>();
 			}
 
 			m_urls[newUrl].append(entry);
@@ -329,6 +367,8 @@ bool HistoryModel::setData(const QModelIndex &index, const QVariant &value, int
 			emit entryModified(entry);
 			emit modelModified();
 
+			break;
+		default:
 			break;
 	}
 
