@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2015 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2017 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,11 +9,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -22,13 +22,37 @@
 namespace Otter
 {
 
-Importer::Importer(QObject *parent) : Addon(parent)
+Importer::Importer(QObject *parent) : QObject(parent), Addon()
 {
 }
 
-QUrl Importer::getUpdateUrl() const
+void Importer::cancel()
 {
-	return QUrl();
+}
+
+void Importer::notifyImportStarted(int type, int total)
+{
+	emit importStarted(static_cast<ImportType>(type), total);
+}
+
+void Importer::notifyImportProgress(int type, int total, int amount)
+{
+	emit importProgress(static_cast<ImportType>(type), total, amount);
+}
+
+void Importer::notifyImportFinished(int type, int result, int total)
+{
+	emit importFinished(static_cast<ImportType>(type), static_cast<ImportResult>(result), total);
+}
+
+Addon::AddonType Importer::getType() const
+{
+	return ImporterType;
+}
+
+bool Importer::canCancel()
+{
+	return false;
 }
 
 }
