@@ -1,7 +1,7 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 * Copyright (C) 2014 Jan Bajer aka bajasoft <jbajer@gmail.com>
+* Copyright (C) 2014 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -10,11 +10,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -22,12 +22,13 @@
 #define OTTER_NETWORKAUTOMATICPROXY_H
 
 #include <QtNetwork/QNetworkProxy>
+#include <QtNetwork/QNetworkReply>
 #include <QtQml/QJSEngine>
 
 namespace Otter
 {
 
-class PacUtils : public QObject
+class PacUtils final : public QObject
 {
 	Q_OBJECT
 
@@ -45,8 +46,8 @@ public slots:
 	bool localHostOrDomainIs(const QString &host, QString domain) const;
 	bool dnsDomainIs(const QString &host, const QString &domain) const;
 	bool shExpMatch(const QString &string, const QString &expression) const;
-	bool weekdayRange(QString fromDay, QString toDay = QString(), const QString &gmt = QLatin1String("gmt")) const;
-	bool dateRange(const QVariant &arg1, const QVariant &arg2 = QVariant(), const QVariant &arg3 = QVariant(), const QVariant &arg4 = QVariant(), const QVariant &arg5 = QVariant(), const QVariant &arg6 = QVariant(), const QString &gmt = QLatin1String("gmt")) const;
+	bool weekdayRange(QString fromDay, QString toDay = {}, const QString &gmt = QLatin1String("gmt")) const;
+	bool dateRange(const QVariant &arg1, const QVariant &arg2 = {}, const QVariant &arg3 = {}, const QVariant &arg4 = {}, const QVariant &arg5 = {}, const QVariant &arg6 = {}, const QString &gmt = QLatin1String("gmt")) const;
 	bool timeRange(const QVariant &arg1, const QVariant &arg2, const QVariant &arg3, const QVariant &arg4, const QVariant &arg5, const QVariant &arg6, const QString &gmt = QLatin1String("gmt")) const;
 
 protected:
@@ -57,20 +58,31 @@ private:
 	static QStringList m_days;
 };
 
-class NetworkAutomaticProxy : public QObject
+class NetworkAutomaticProxy final : public QObject
 {
 	Q_OBJECT
 
 public:
-	explicit NetworkAutomaticProxy(QObject *parent = nullptr);
+	explicit NetworkAutomaticProxy(const QString &path, QObject *parent = nullptr);
 
-	QList<QNetworkProxy> getProxy(const QString &url, const QString &host);
+	void setPath(const QString &path);
+	QString getPath() const;
+	QVector<QNetworkProxy> getProxy(const QString &url, const QString &host);
+	bool isValid() const;
+
+protected:
 	bool setup(const QString &script);
 
+protected slots:
+	void handleReplyFinished();
+
 private:
+	QNetworkReply *m_reply;
 	QJSEngine m_engine;
 	QJSValue m_findProxy;
-	QHash<QString, QList<QNetworkProxy> > m_proxies;
+	QString m_path;
+	QHash<QString, QVector<QNetworkProxy> > m_proxies;
+	bool m_isValid;
 };
 
 }
