@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2014 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 * Copyright (C) 2015 Jan Bajer aka bajasoft <jbajer@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
@@ -10,11 +10,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -22,30 +22,43 @@
 #include "Application.h"
 #include "Console.h"
 #include "Updater.h"
-#include "Utils.h"
 
 #include <QtCore/QDir>
+#include <QtCore/QMimeData>
 #include <QtCore/QProcess>
+#include <QtGui/QDrag>
 #include <QtWidgets/QMessageBox>
 
 namespace Otter
 {
 
-PlatformIntegration::PlatformIntegration(Application *parent) : QObject(parent)
+PlatformIntegration::PlatformIntegration(QObject *parent) : QObject(parent)
 {
 }
 
-void PlatformIntegration::runApplication(const QString &command, const QUrl &url) const
+
+void PlatformIntegration::startLinkDrag(const QUrl &url, const QString &title, const QPixmap &pixmap, QObject *parent) const
 {
-	Q_UNUSED(command)
-	Q_UNUSED(url)
+	QDrag *drag(new QDrag(parent));
+	QMimeData *mimeData(new QMimeData());
+	mimeData->setText(url.toString());
+	mimeData->setUrls({url});
+
+	if (!title.isEmpty())
+	{
+		mimeData->setProperty("x-url-title", title);
+	}
+
+	mimeData->setProperty("x-url-string", url.toString());
+
+	drag->setMimeData(mimeData);
+	drag->setPixmap(pixmap);
+	drag->exec(Qt::CopyAction);
 }
 
-QList<ApplicationInformation> PlatformIntegration::getApplicationsForMimeType(const QMimeType &mimeType)
+QString PlatformIntegration::getPreferredPasswordsBackend() const
 {
-	Q_UNUSED(mimeType)
-
-	return QList<ApplicationInformation>();
+	return QLatin1String("file");
 }
 
 QString PlatformIntegration::getUpdaterBinary() const
@@ -53,9 +66,9 @@ QString PlatformIntegration::getUpdaterBinary() const
 	return QLatin1String("updater");
 }
 
-QString PlatformIntegration::getPlatform() const
+QString PlatformIntegration::getPlatformName() const
 {
-	return QString();
+	return {};
 }
 
 bool PlatformIntegration::canShowNotifications() const
@@ -85,12 +98,12 @@ void PlatformIntegration::showNotification(Notification *notification)
 
 bool PlatformIntegration::installUpdate() const
 {
-	const QString updaterPath = QDir::toNativeSeparators(QCoreApplication::applicationDirPath() + QDir::separator() + getUpdaterBinary());
-	const QString scriptPath = Updater::getScriptPath();
+	const QString updaterPath(QDir::toNativeSeparators(QCoreApplication::applicationDirPath() + QDir::separator() + getUpdaterBinary()));
+	const QString scriptPath(Updater::getScriptPath());
 
 	if (Updater::isReadyToInstall(scriptPath) && QFileInfo(updaterPath).isExecutable())
 	{
-		if (QProcess::startDetached(updaterPath, QStringList() << QLatin1String("--install-dir") << QCoreApplication::applicationDirPath() << QLatin1String("--package-dir") << QFileInfo(scriptPath).absolutePath() << QLatin1String("--script") << scriptPath))
+		if (QProcess::startDetached(updaterPath, {QLatin1String("--install-dir"), QCoreApplication::applicationDirPath(), QLatin1String("--package-dir"), QFileInfo(scriptPath).absolutePath(), QLatin1String("--script"), scriptPath}))
 		{
 			Updater::clearUpdate();
 
@@ -100,9 +113,9 @@ bool PlatformIntegration::installUpdate() const
 		}
 	}
 
-	Console::addMessage(QCoreApplication::translate("main", "Failed to install update\nUpdater: %1\nScript: %2").arg(updaterPath).arg(scriptPath), OtherMessageCategory, ErrorMessageLevel);
+	Console::addMessage(QCoreApplication::translate("main", "Failed to install update\nUpdater: %1\nScript: %2").arg(updaterPath).arg(scriptPath), Console::OtherCategory, Console::ErrorLevel);
 
-	QMessageBox::critical(NULL, tr("Error"), tr("Failed to install update."), QMessageBox::Close);
+	QMessageBox::critical(nullptr, tr("Error"), tr("Failed to install update."), QMessageBox::Close);
 
 	Updater::clearUpdate();
 
