@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,15 +9,16 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
 #include "TransfersManager.h"
+#include "Application.h"
 #include "NetworkManager.h"
 #include "NetworkManagerFactory.h"
 #include "NotificationsManager.h"
@@ -31,15 +32,17 @@
 #include <QtCore/QStandardPaths>
 #include <QtCore/QTemporaryFile>
 #include <QtCore/QTimer>
+#include <QtNetwork/QAbstractNetworkCache>
 #include <QtWidgets/QMessageBox>
 
 namespace Otter
 {
 
 TransfersManager* TransfersManager::m_instance(nullptr);
-QList<Transfer*> TransfersManager::m_transfers;
-QList<Transfer*> TransfersManager::m_privateTransfers;
+QVector<Transfer*> TransfersManager::m_transfers;
+QVector<Transfer*> TransfersManager::m_privateTransfers;
 bool TransfersManager::m_isInitilized(false);
+bool TransfersManager::m_hasRunningTransfers(false);
 
 Transfer::Transfer(TransferOptions options, QObject *parent) : QObject(parent ? parent : TransfersManager::getInstance()),
 	m_reply(nullptr),
@@ -53,7 +56,9 @@ Transfer::Transfer(TransferOptions options, QObject *parent) : QObject(parent ?
 	m_state(UnknownState),
 	m_updateTimer(0),
 	m_updateInterval(0),
-	m_isSelectingPath(false)
+	m_remainingTime(-1),
+	m_isSelectingPath(false),
+	m_isArchived(false)
 {
 }
 
@@ -71,11 +76,15 @@ Transfer::Transfer(const QSettings &settings, QObject *parent) : QObject(parent
 	m_bytesReceived(settings.value(QLatin1String("bytesReceived")).toLongLong()),
 	m_bytesTotal(settings.value(QLatin1String("bytesTotal")).toLongLong()),
 	m_options(NoOption),
-	m_state((m_bytesReceived > 0 && m_bytesTotal == m_bytesReceived) ? FinishedState : ErrorState),
+	m_state((m_bytesReceived > 0 && m_bytesTotal == m_bytesReceived && QFile::exists(settings.value(QLatin1String("target")).toString())) ? FinishedState : ErrorState),
 	m_updateTimer(0),
 	m_updateInterval(0),
-	m_isSelectingPath(false)
+	m_remainingTime(-1),
+	m_isSelectingPath(false),
+	m_isArchived(true)
 {
+	m_timeStarted.setTimeSpec(Qt::UTC);
+	m_timeFinished.setTimeSpec(Qt::UTC);
 }
 
 Transfer::Transfer(const QUrl &source, const QString &target, TransferOptions options, QObject *parent) : QObject(parent ? parent : TransfersManager::getInstance()),
@@ -92,14 +101,17 @@ Transfer::Transfer(const QUrl &source, const QString &target, TransferOptions op
 	m_state(UnknownState),
 	m_updateTimer(0),
 	m_updateInterval(0),
-	m_isSelectingPath(false)
+	m_remainingTime(-1),
+	m_isSelectingPath(false),
+	m_isArchived(false)
 {
 	QNetworkRequest request;
 	request.setAttribute(QNetworkRequest::CacheLoadControlAttribute, QNetworkRequest::AlwaysNetwork);
+	request.setAttribute(QNetworkRequest::FollowRedirectsAttribute, true);
 	request.setHeader(QNetworkRequest::UserAgentHeader, NetworkManagerFactory::getUserAgent());
 	request.setUrl(QUrl(source));
 
-	start(NetworkManagerFactory::getNetworkManager()->get(request), target);
+	start(NetworkManagerFactory::getNetworkManager(m_options.testFlag(IsPrivateOption))->get(request), target);
 }
 
 Transfer::Transfer(const QNetworkRequest &request, const QString &target, TransferOptions options, QObject *parent) : QObject(parent ? parent : TransfersManager::getInstance()),
@@ -116,14 +128,16 @@ Transfer::Transfer(const QNetworkRequest &request, const QString &target, Transf
 	m_state(UnknownState),
 	m_updateTimer(0),
 	m_updateInterval(0),
-	m_isSelectingPath(false)
+	m_remainingTime(-1),
+	m_isSelectingPath(false),
+	m_isArchived(false)
 {
-	start(NetworkManagerFactory::getNetworkManager()->get(request), target);
+	start(NetworkManagerFactory::getNetworkManager(m_options.testFlag(IsPrivateOption))->get(request), target);
 }
 
 Transfer::Transfer(QNetworkReply *reply, const QString &target, TransferOptions options, QObject *parent) : QObject(parent ? parent : TransfersManager::getInstance()),
 	m_reply(reply),
-	m_source(m_reply->url().adjusted(QUrl::RemovePassword | QUrl::PreferLocalFile)),
+	m_source((m_reply->url().isValid() ? m_reply->url() : m_reply->request().url()).adjusted(QUrl::RemovePassword | QUrl::PreferLocalFile)),
 	m_target(target),
 	m_speed(0),
 	m_bytesStart(0),
@@ -134,7 +148,9 @@ Transfer::Transfer(QNetworkReply *reply, const QString &target, TransferOptions
 	m_state(UnknownState),
 	m_updateTimer(0),
 	m_updateInterval(0),
-	m_isSelectingPath(false)
+	m_remainingTime(-1),
+	m_isSelectingPath(false),
+	m_isArchived(false)
 {
 	start(reply, target);
 }
@@ -158,6 +174,27 @@ void Transfer::timerEvent(QTimerEvent *event)
 
 		if (m_speed != oldSpeed)
 		{
+			m_speeds.enqueue(m_speed);
+
+			if (m_speeds.count() > 10)
+			{
+				m_speeds.dequeue();
+			}
+
+			if (m_bytesTotal > 0)
+			{
+				qint64 speedSum(0);
+
+				for (int i = 0; i < m_speeds.count(); ++i)
+				{
+					speedSum += m_speeds.at(i);
+				}
+
+				speedSum /= m_speeds.count();
+
+				m_remainingTime = qRound(static_cast<qreal>(m_bytesTotal - m_bytesReceived) / static_cast<qreal>(speedSum));
+			}
+
 			emit changed();
 		}
 	}
@@ -177,8 +214,10 @@ void Transfer::start(QNetworkReply *reply, const QString &target)
 		return;
 	}
 
+	const QMimeDatabase mimeDatabase;
+
 	m_reply = reply;
-	m_mimeType = QMimeDatabase().mimeTypeForName(m_reply->header(QNetworkRequest::ContentTypeHeader).toString());
+	m_mimeType = mimeDatabase.mimeTypeForName(m_reply->header(QNetworkRequest::ContentTypeHeader).toString());
 
 	QString temporaryFileName(getSuggestedFileName());
 
@@ -188,16 +227,34 @@ void Transfer::start(QNetworkReply *reply, const QString &target)
 	}
 	else if (temporaryFileName.contains(QLatin1Char('.')))
 	{
-		QStringList parts(temporaryFileName.split(QLatin1Char('.')));
-		parts[parts.count() - 2].append(QLatin1String("-XXXXXX"));
+		const QString suffix(mimeDatabase.suffixForFileName(temporaryFileName.simplified().remove(QLatin1Char(' '))));
+		int position(temporaryFileName.lastIndexOf(QLatin1Char('.')));
 
-		temporaryFileName = parts.join(QLatin1Char('.'));
+		if (!suffix.isEmpty() && temporaryFileName.endsWith(suffix, Qt::CaseInsensitive))
+		{
+			position = (temporaryFileName.length() - suffix.length() - 1);
+		}
+
+		temporaryFileName = temporaryFileName.insert(position, QLatin1String("-XXXXXX"));
 	}
 
 	m_device = new QTemporaryFile(QStandardPaths::writableLocation(QStandardPaths::TempLocation) + QDir::separator() + temporaryFileName, this);
-	m_timeStarted = QDateTime::currentDateTime();
+	m_timeStarted = QDateTime::currentDateTimeUtc();
 	m_bytesTotal = m_reply->header(QNetworkRequest::ContentLengthHeader).toLongLong();
 
+	if (m_bytesTotal == 0 && reply->attribute(QNetworkRequest::SourceIsFromCacheAttribute).toBool() && reply->manager()->cache())
+	{
+		QIODevice *device(reply->manager()->cache()->data(m_source));
+
+		if (device)
+		{
+			m_bytesTotal = device->size();
+
+			device->close();
+			device->deleteLater();
+		}
+	}
+
 	if (!m_device->open(QIODevice::ReadWrite))
 	{
 		m_state = ErrorState;
@@ -213,19 +270,19 @@ void Transfer::start(QNetworkReply *reply, const QString &target)
 	m_target = m_device->fileName();
 	m_state = (m_reply->isFinished() ? FinishedState : RunningState);
 
-	downloadData();
+	handleDataAvailable();
 
 	const bool isRunning(m_state == RunningState);
 
 	if (isRunning)
 	{
-		connect(m_reply, SIGNAL(downloadProgress(qint64,qint64)), this, SLOT(downloadProgress(qint64,qint64)));
-		connect(m_reply, SIGNAL(finished()), this, SLOT(downloadFinished()));
-		connect(m_reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(downloadError(QNetworkReply::NetworkError)));
+		connect(m_reply, &QNetworkReply::downloadProgress, this, &Transfer::handleDownloadProgress);
+		connect(m_reply, &QNetworkReply::finished, this, &Transfer::handleDownloadFinished);
+		connect(m_reply, static_cast<void(QNetworkReply::*)(QNetworkReply::NetworkError)>(&QNetworkReply::error), this, &Transfer::handleDownloadError);
 	}
 	else
 	{
-		markFinished();
+		markAsFinished();
 
 		if (m_options.testFlag(CanAutoDeleteOption) && !m_isSelectingPath)
 		{
@@ -235,15 +292,15 @@ void Transfer::start(QNetworkReply *reply, const QString &target)
 
 	m_device->reset();
 
-	m_mimeType = QMimeDatabase().mimeTypeForData(m_device);
+	m_mimeType = mimeDatabase.mimeTypeForData(m_device);
 
 	m_device->seek(m_device->size());
 
-	downloadData();
+	handleDataAvailable();
 
 	if (isRunning)
 	{
-		connect(m_reply, SIGNAL(readyRead()), this, SLOT(downloadData()));
+		connect(m_reply, &QNetworkReply::readyRead, this, &Transfer::handleDataAvailable);
 	}
 
 	QString finalTarget;
@@ -251,21 +308,21 @@ void Transfer::start(QNetworkReply *reply, const QString &target)
 
 	if (target.isEmpty())
 	{
-		QString path;
-		QString fileName(getSuggestedFileName());
-
-		if (!SettingsManager::getValue(SettingsManager::Browser_AlwaysAskWhereToSaveDownloadOption).toBool())
+		if (!SettingsManager::getOption(SettingsManager::Browser_AlwaysAskWhereToSaveDownloadOption).toBool())
 		{
 			m_options |= IsQuickTransferOption;
 		}
 
+		const QString directory(m_options.testFlag(IsQuickTransferOption) ? Utils::normalizePath(SettingsManager::getOption(SettingsManager::Paths_DownloadsOption).toString()) : QString());
+		const QString fileName(getSuggestedFileName());
+
 		if (m_options.testFlag(IsQuickTransferOption))
 		{
-			path = SettingsManager::getValue(SettingsManager::Paths_DownloadsOption).toString();
+			finalTarget = directory + QDir::separator() + fileName;
 
-			if (QFile::exists(path + QDir::separator() + fileName) && QMessageBox::question(SessionsManager::getActiveWindow(), tr("Question"), tr("File with that name already exists.\nDo you want to overwrite it?"), (QMessageBox::Yes | QMessageBox::No)) == QMessageBox::No)
+			if (QFile::exists(finalTarget))
 			{
-				path = QString();
+				m_options |= CanAskForPathOption;
 			}
 		}
 
@@ -273,14 +330,11 @@ void Transfer::start(QNetworkReply *reply, const QString &target)
 		{
 			m_isSelectingPath = true;
 
-			const SaveInformation information(Utils::getSavePath(fileName, path));
-
-			path = information.path;
-			canOverwriteExisting = information.canOverwriteExisting;
+			const SaveInformation information(Utils::getSavePath(fileName, directory));
 
 			m_isSelectingPath = false;
 
-			if (path.isEmpty())
+			if (!information.canSave)
 			{
 				if (m_reply)
 				{
@@ -293,9 +347,12 @@ void Transfer::start(QNetworkReply *reply, const QString &target)
 
 				return;
 			}
+
+			finalTarget = information.path;
+			canOverwriteExisting = true;
 		}
 
-		finalTarget = QDir::toNativeSeparators(path);
+		finalTarget = QDir::toNativeSeparators(finalTarget);
 	}
 	else
 	{
@@ -325,160 +382,12 @@ void Transfer::start(QNetworkReply *reply, const QString &target)
 		}
 		else
 		{
-			m_mimeType = QMimeDatabase().mimeTypeForFile(m_target);
+			m_mimeType = mimeDatabase.mimeTypeForFile(m_target);
 		}
 	}
 }
 
-void Transfer::downloadProgress(qint64 bytesReceived, qint64 bytesTotal)
-{
-	m_bytesReceivedDifference += (bytesReceived - (m_bytesReceived - m_bytesStart));
-	m_bytesReceived = (m_bytesStart + bytesReceived);
-	m_bytesTotal = (m_bytesStart + bytesTotal);
-
-	emit progressChanged(bytesReceived, bytesTotal);
-}
-
-void Transfer::downloadData()
-{
-	if (!m_reply || !m_device)
-	{
-		return;
-	}
-
-	if (m_state == ErrorState)
-	{
-		m_state = RunningState;
-
-		if (m_reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).isValid() && m_reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt() != 206)
-		{
-			m_device->reset();
-		}
-	}
-
-	m_device->write(m_reply->readAll());
-	m_device->seek(m_device->size());
-
-	if (m_state == RunningState && m_reply->attribute(QNetworkRequest::SourceIsFromCacheAttribute).toBool() && m_bytesTotal >= 0 && m_device->size() == m_bytesTotal)
-	{
-		downloadFinished();
-	}
-}
-
-void Transfer::downloadFinished()
-{
-	if (!m_reply)
-	{
-		if (m_device && !m_device->inherits(QStringLiteral("QTemporaryFile").toLatin1()))
-		{
-			m_device->close();
-			m_device->deleteLater();
-			m_device = nullptr;
-		}
-
-		if (m_options.testFlag(CanAutoDeleteOption) && !m_isSelectingPath)
-		{
-			deleteLater();
-		}
-
-		return;
-	}
-
-	if (!m_reply->attribute(QNetworkRequest::RedirectionTargetAttribute).isNull())
-	{
-		m_source = m_source.resolved(m_reply->attribute(QNetworkRequest::RedirectionTargetAttribute).toUrl());
-
-		restart();
-
-		return;
-	}
-
-	if (m_updateTimer != 0)
-	{
-		killTimer(m_updateTimer);
-
-		m_updateTimer = 0;
-	}
-
-	if (m_reply->size() > 0)
-	{
-		m_device->write(m_reply->readAll());
-	}
-
-	disconnect(m_reply, SIGNAL(downloadProgress(qint64,qint64)), this, SLOT(downloadProgress(qint64,qint64)));
-	disconnect(m_reply, SIGNAL(readyRead()), this, SLOT(downloadData()));
-	disconnect(m_reply, SIGNAL(finished()), this, SLOT(downloadFinished()));
-
-	m_bytesReceived = (m_device ? m_device->size() : -1);
-
-	if (m_bytesTotal <= 0 && m_bytesReceived > 0)
-	{
-		m_bytesTotal = m_bytesReceived;
-	}
-
-	if (m_bytesReceived == 0 || m_bytesReceived < m_bytesTotal)
-	{
-		m_state = ErrorState;
-	}
-	else
-	{
-		markFinished();
-
-		m_state = FinishedState;
-		m_mimeType = QMimeDatabase().mimeTypeForFile(m_target);
-	}
-
-	emit finished();
-	emit changed();
-
-	if (m_device && (m_options.testFlag(HasToOpenAfterFinishOption) || !m_device->inherits(QStringLiteral("QTemporaryFile").toLatin1())))
-	{
-		m_device->close();
-		m_device->deleteLater();
-		m_device = nullptr;
-
-		if (m_reply)
-		{
-			QTimer::singleShot(250, m_reply, SLOT(deleteLater()));
-		}
-	}
-
-	if (m_state == FinishedState && m_options.testFlag(HasToOpenAfterFinishOption))
-	{
-		openTarget();
-	}
-
-	if (m_options.testFlag(CanAutoDeleteOption) && !m_isSelectingPath)
-	{
-		deleteLater();
-	}
-}
-
-void Transfer::downloadError(QNetworkReply::NetworkError error)
-{
-	Q_UNUSED(error)
-
-	stop();
-
-	m_state = ErrorState;
-
-	if (m_options.testFlag(CanAutoDeleteOption) && !m_isSelectingPath)
-	{
-		deleteLater();
-	}
-}
-
-void Transfer::markStarted()
-{
-	m_timeStarted = QDateTime::currentDateTime();
-}
-
-void Transfer::markFinished(bool reset)
-{
-	m_timeFinished = (reset ? QDateTime() : QDateTime::currentDateTime());
-}
-
-void Transfer::openTarget()
+void Transfer::openTarget() const
 {
 	Utils::runApplication(m_openCommand, QUrl::fromLocalFile(getTarget()));
 }
@@ -491,7 +400,7 @@ void Transfer::cancel()
 	{
 		m_reply->abort();
 
-		QTimer::singleShot(250, m_reply, SLOT(deleteLater()));
+		QTimer::singleShot(250, m_reply, &QNetworkReply::deleteLater);
 	}
 
 	if (m_device)
@@ -520,10 +429,10 @@ void Transfer::stop()
 	{
 		m_reply->abort();
 
-		QTimer::singleShot(250, m_reply, SLOT(deleteLater()));
+		QTimer::singleShot(250, m_reply, &QNetworkReply::deleteLater);
 	}
 
-	if (m_device && !m_device->inherits(QStringLiteral("QTemporaryFile").toLatin1()))
+	if (m_device && !m_device->inherits("QTemporaryFile"))
 	{
 		m_device->close();
 		m_device->deleteLater();
@@ -540,10 +449,169 @@ void Transfer::stop()
 		}
 	}
 
+	m_speeds.clear();
+
 	emit stopped();
 	emit changed();
 }
 
+void Transfer::markAsStarted()
+{
+	m_timeStarted = QDateTime::currentDateTimeUtc();
+}
+
+void Transfer::markAsFinished()
+{
+	m_timeFinished = QDateTime::currentDateTimeUtc();
+}
+
+void Transfer::handleDownloadProgress(qint64 bytesReceived, qint64 bytesTotal)
+{
+	m_bytesReceivedDifference += (bytesReceived - (m_bytesReceived - m_bytesStart));
+	m_bytesReceived = (m_bytesStart + bytesReceived);
+	m_bytesTotal = (m_bytesStart + bytesTotal);
+
+	emit progressChanged(bytesReceived, bytesTotal);
+}
+
+void Transfer::handleDataAvailable()
+{
+	if (!m_reply || !m_device)
+	{
+		return;
+	}
+
+	if (m_state == ErrorState)
+	{
+		m_state = RunningState;
+
+		if (m_reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).isValid() && m_reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt() != 206)
+		{
+			m_device->reset();
+		}
+	}
+
+	m_device->write(m_reply->readAll());
+	m_device->seek(m_device->size());
+
+	if (m_state == RunningState && m_reply->attribute(QNetworkRequest::SourceIsFromCacheAttribute).toBool() && m_bytesTotal >= 0 && m_device->size() == m_bytesTotal)
+	{
+		handleDownloadFinished();
+	}
+}
+
+void Transfer::handleDownloadFinished()
+{
+	if (!m_reply)
+	{
+		if (m_device && !m_device->inherits("QTemporaryFile"))
+		{
+			m_device->close();
+			m_device->deleteLater();
+			m_device = nullptr;
+		}
+
+		if (m_options.testFlag(CanAutoDeleteOption) && !m_isSelectingPath)
+		{
+			deleteLater();
+		}
+
+		return;
+	}
+
+	if (!m_reply->attribute(QNetworkRequest::RedirectionTargetAttribute).isNull())
+	{
+		const QUrl url(m_source.resolved(m_reply->attribute(QNetworkRequest::RedirectionTargetAttribute).toUrl()));
+
+		if (!url.isValid() || (m_source.scheme() == QLatin1String("https") && url.scheme() == QLatin1String("http")))
+		{
+			handleDownloadError(QNetworkReply::UnknownContentError);
+		}
+		else
+		{
+			m_source = url;
+
+			restart();
+		}
+
+		return;
+	}
+
+	if (m_updateTimer != 0)
+	{
+		killTimer(m_updateTimer);
+
+		m_updateTimer = 0;
+	}
+
+	if (m_reply->size() > 0)
+	{
+		m_device->write(m_reply->readAll());
+	}
+
+	disconnect(m_reply, &QNetworkReply::downloadProgress, this, &Transfer::handleDownloadProgress);
+	disconnect(m_reply, &QNetworkReply::readyRead, this, &Transfer::handleDataAvailable);
+	disconnect(m_reply, &QNetworkReply::finished, this, &Transfer::handleDownloadFinished);
+
+	m_bytesReceived = (m_device ? m_device->size() : -1);
+
+	if (m_bytesTotal <= 0 && m_bytesReceived > 0)
+	{
+		m_bytesTotal = m_bytesReceived;
+	}
+
+	if (m_bytesReceived == 0 || m_bytesReceived < m_bytesTotal)
+	{
+		m_state = ErrorState;
+	}
+	else
+	{
+		markAsFinished();
+
+		m_state = FinishedState;
+		m_mimeType = QMimeDatabase().mimeTypeForFile(m_target);
+	}
+
+	emit finished();
+	emit changed();
+
+	if (m_device && (m_options.testFlag(HasToOpenAfterFinishOption) || !m_device->inherits("QTemporaryFile")))
+	{
+		m_device->close();
+		m_device->deleteLater();
+		m_device = nullptr;
+
+		if (m_reply)
+		{
+			QTimer::singleShot(250, m_reply, &QNetworkReply::deleteLater);
+		}
+	}
+
+	if (m_state == FinishedState && m_options.testFlag(HasToOpenAfterFinishOption))
+	{
+		openTarget();
+	}
+
+	if (m_options.testFlag(CanAutoDeleteOption) && !m_isSelectingPath)
+	{
+		deleteLater();
+	}
+}
+
+void Transfer::handleDownloadError(QNetworkReply::NetworkError error)
+{
+	Q_UNUSED(error)
+
+	stop();
+
+	m_state = ErrorState;
+
+	if (m_options.testFlag(CanAutoDeleteOption) && !m_isSelectingPath)
+	{
+		deleteLater();
+	}
+}
+
 void Transfer::setOpenCommand(const QString &command)
 {
 	m_openCommand = command;
@@ -597,31 +665,35 @@ QString Transfer::getSuggestedFileName()
 {
 	if (!m_reply)
 	{
-		return m_suggestedFileName;
+		return (m_suggestedFileName.isEmpty() ? tr("file") : m_suggestedFileName);
 	}
 
-	QUrl url;
 	QString fileName;
 
 	if (m_reply->hasRawHeader(QStringLiteral("Content-Disposition").toLatin1()))
 	{
-		fileName = QRegularExpression(QLatin1String(" filename=[\"]?([^\"]+)[\"]?(; )?")).match(QString(m_reply->rawHeader(QStringLiteral("Content-Disposition").toLatin1()))).captured(1);
+		const QString contenDispositionHeader(m_reply->rawHeader(QStringLiteral("Content-Disposition").toLatin1()));
+
+		if (contenDispositionHeader.contains(QLatin1String("filename*=")))
+		{
+			fileName = QRegularExpression(QLatin1String("[\\s;]filename\\*=[\"]?[a-zA-Z0-9\\-_]+\\'[a-zA-Z0-9\\-]?\\'([^\"]+)[\"]?[\\s;]?")).match(contenDispositionHeader).captured(1);
+		}
+		else
+		{
+			fileName = QRegularExpression(QLatin1String("[\\s;]filename=[\"]?([^\"]+)[\"]?[\\s;]?")).match(contenDispositionHeader).captured(1);
+		}
 
 		if (fileName.contains(QLatin1String("; ")))
 		{
 			fileName = fileName.section(QLatin1String("; "), 0, 0);
 		}
 
-		url = QUrl(fileName);
-
-		fileName = url.fileName();
+		fileName = QUrl(fileName).fileName();
 	}
 
 	if (fileName.isEmpty())
 	{
-		url = m_source;
-
-		fileName = url.fileName();
+		fileName = m_source.fileName();
 	}
 
 	if (fileName.isEmpty())
@@ -695,6 +767,16 @@ Transfer::TransferState Transfer::getState() const
 	return m_state;
 }
 
+int Transfer::getRemainingTime() const
+{
+	return m_remainingTime;
+}
+
+bool Transfer::isArchived() const
+{
+	return m_isArchived;
+}
+
 bool Transfer::resume()
 {
 	if (m_state != ErrorState || !QFile::exists(m_target))
@@ -702,6 +784,8 @@ bool Transfer::resume()
 		return false;
 	}
 
+	m_isArchived = false;
+
 	if (m_bytesTotal == 0)
 	{
 		return restart();
@@ -718,8 +802,8 @@ bool Transfer::resume()
 
 	m_state = RunningState;
 	m_device = file;
-	m_timeStarted = QDateTime::currentDateTime();
-	m_timeFinished = QDateTime();
+	m_timeStarted = QDateTime::currentDateTimeUtc();
+	m_timeFinished = {};
 	m_bytesStart = file->size();
 
 	QNetworkRequest request;
@@ -728,14 +812,14 @@ bool Transfer::resume()
 	request.setRawHeader(QStringLiteral("Range").toLatin1(), QStringLiteral("bytes=%1-").arg(file->size()).toLatin1());
 	request.setUrl(m_source);
 
-	m_reply = NetworkManagerFactory::getNetworkManager()->get(request);
+	m_reply = NetworkManagerFactory::getNetworkManager(m_options.testFlag(IsPrivateOption))->get(request);
 
-	downloadData();
+	handleDataAvailable();
 
-	connect(m_reply, SIGNAL(downloadProgress(qint64,qint64)), this, SLOT(downloadProgress(qint64,qint64)));
-	connect(m_reply, SIGNAL(readyRead()), this, SLOT(downloadData()));
-	connect(m_reply, SIGNAL(finished()), this, SLOT(downloadFinished()));
-	connect(m_reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(downloadError(QNetworkReply::NetworkError)));
+	connect(m_reply, &QNetworkReply::downloadProgress, this, &Transfer::handleDownloadProgress);
+	connect(m_reply, &QNetworkReply::readyRead, this, &Transfer::handleDataAvailable);
+	connect(m_reply, &QNetworkReply::finished, this, &Transfer::handleDownloadFinished);
+	connect(m_reply, static_cast<void(QNetworkReply::*)(QNetworkReply::NetworkError)>(&QNetworkReply::error), this, &Transfer::handleDownloadError);
 
 	if (m_updateTimer == 0 && m_updateInterval > 0)
 	{
@@ -749,6 +833,8 @@ bool Transfer::restart()
 {
 	stop();
 
+	m_isArchived = false;
+
 	QFile *file(new QFile(m_target));
 
 	if (!file->open(QIODevice::WriteOnly))
@@ -760,23 +846,23 @@ bool Transfer::restart()
 
 	m_state = RunningState;
 	m_device = file;
-	m_timeStarted = QDateTime::currentDateTime();
-	m_timeFinished = QDateTime();
+	m_timeStarted = QDateTime::currentDateTimeUtc();
+	m_timeFinished = {};
 	m_bytesStart = 0;
 
 	QNetworkRequest request;
 	request.setAttribute(QNetworkRequest::CacheLoadControlAttribute, QNetworkRequest::AlwaysNetwork);
 	request.setHeader(QNetworkRequest::UserAgentHeader, NetworkManagerFactory::getUserAgent());
-	request.setUrl(QUrl(m_source));
+	request.setUrl(m_source);
 
-	m_reply = NetworkManagerFactory::getNetworkManager()->get(request);
+	m_reply = NetworkManagerFactory::getNetworkManager(m_options.testFlag(IsPrivateOption))->get(request);
 
-	downloadData();
+	handleDataAvailable();
 
-	connect(m_reply, SIGNAL(downloadProgress(qint64,qint64)), this, SLOT(downloadProgress(qint64,qint64)));
-	connect(m_reply, SIGNAL(readyRead()), this, SLOT(downloadData()));
-	connect(m_reply, SIGNAL(finished()), this, SLOT(downloadFinished()));
-	connect(m_reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(downloadError(QNetworkReply::NetworkError)));
+	connect(m_reply, &QNetworkReply::downloadProgress, this, &Transfer::handleDownloadProgress);
+	connect(m_reply, &QNetworkReply::readyRead, this, &Transfer::handleDataAvailable);
+	connect(m_reply, &QNetworkReply::finished, this, &Transfer::handleDownloadFinished);
+	connect(m_reply, static_cast<void(QNetworkReply::*)(QNetworkReply::NetworkError)>(&QNetworkReply::error), this, &Transfer::handleDownloadError);
 
 	if (m_updateTimer == 0 && m_updateInterval > 0)
 	{
@@ -797,12 +883,12 @@ bool Transfer::setTarget(const QString &target, bool canOverwriteExisting)
 
 	if (!canOverwriteExisting && !m_options.testFlag(CanOverwriteOption) && QFile::exists(target))
 	{
-		const QMessageBox::StandardButton result(QMessageBox::question(SessionsManager::getActiveWindow(), tr("Question"), tr("File with the same name already exists.\nDo you want to overwrite it?\n\n%1").arg(target), (QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel)));
+		const int result(QMessageBox::question(Application::getActiveWindow(), tr("Question"), tr("File with the same name already exists.\nDo you want to overwrite it?\n\n%1").arg(target), QMessageBox::Yes, QMessageBox::No, QMessageBox::Cancel));
 
 		if (result == QMessageBox::No)
 		{
-			QFileInfo information(target);
-			const QString path(Utils::getSavePath(information.fileName(), information.path(), QStringList(), true).path);
+			const QFileInfo information(target);
+			const QString path(Utils::getSavePath(information.fileName(), information.path(), {}, true).path);
 
 			if (path.isEmpty())
 			{
@@ -860,7 +946,7 @@ bool Transfer::setTarget(const QString &target, bool canOverwriteExisting)
 	{
 		if (m_reply && m_state == RunningState)
 		{
-			disconnect(m_reply, SIGNAL(readyRead()), this, SLOT(downloadData()));
+			disconnect(m_reply, &QNetworkReply::readyRead, this, &Transfer::handleDataAvailable);
 		}
 
 		m_device->reset();
@@ -873,15 +959,15 @@ bool Transfer::setTarget(const QString &target, bool canOverwriteExisting)
 
 	m_device = file;
 
-	downloadData();
+	handleDataAvailable();
 
 	if (!m_reply || m_reply->isFinished())
 	{
-		downloadFinished();
+		handleDownloadFinished();
 	}
 	else
 	{
-		connect(m_reply, SIGNAL(readyRead()), this, SLOT(downloadData()));
+		connect(m_reply, &QNetworkReply::readyRead, this, &Transfer::handleDataAvailable);
 	}
 
 	return false;
@@ -892,11 +978,11 @@ TransfersManager::TransfersManager(QObject *parent) : QObject(parent),
 {
 }
 
-void TransfersManager::createInstance(QObject *parent)
+void TransfersManager::createInstance()
 {
 	if (!m_instance)
 	{
-		m_instance = new TransfersManager(parent);
+		m_instance = new TransfersManager(QCoreApplication::instance());
 	}
 }
 
@@ -920,16 +1006,33 @@ void TransfersManager::scheduleSave()
 	}
 }
 
+void TransfersManager::updateRunningTransfersState()
+{
+	bool hasRunningTransfers(false);
+
+	for (int i = 0; i < m_transfers.count(); ++i)
+	{
+		if (m_transfers.at(i)->getState() == Transfer::RunningState)
+		{
+			hasRunningTransfers = true;
+
+			break;
+		}
+	}
+
+	m_hasRunningTransfers = hasRunningTransfers;
+}
+
 void TransfersManager::addTransfer(Transfer *transfer)
 {
 	m_transfers.append(transfer);
 
 	transfer->setUpdateInterval(500);
 
-	connect(transfer, SIGNAL(started()), m_instance, SLOT(transferStarted()));
-	connect(transfer, SIGNAL(finished()), m_instance, SLOT(transferFinished()));
-	connect(transfer, SIGNAL(changed()), m_instance, SLOT(transferChanged()));
-	connect(transfer, SIGNAL(stopped()), m_instance, SLOT(transferStopped()));
+	connect(transfer, &Transfer::started, m_instance, &TransfersManager::handleTransferStarted);
+	connect(transfer, &Transfer::finished, m_instance, &TransfersManager::handleTransferFinished);
+	connect(transfer, &Transfer::changed, m_instance, &TransfersManager::handleTransferChanged);
+	connect(transfer, &Transfer::stopped, m_instance, &TransfersManager::handleTransferStopped);
 
 	if (transfer->getOptions().testFlag(Transfer::CanNotifyOption) && transfer->getState() != Transfer::CancelledState)
 	{
@@ -949,7 +1052,7 @@ void TransfersManager::addTransfer(Transfer *transfer)
 
 void TransfersManager::save()
 {
-	if (SessionsManager::isReadOnly() || SettingsManager::getValue(SettingsManager::Browser_PrivateModeOption).toBool() || !SettingsManager::getValue(SettingsManager::History_RememberDownloadsOption).toBool())
+	if (SessionsManager::isReadOnly() || SettingsManager::getOption(SettingsManager::Browser_PrivateModeOption).toBool() || !SettingsManager::getOption(SettingsManager::History_RememberDownloadsOption).toBool())
 	{
 		return;
 	}
@@ -957,12 +1060,12 @@ void TransfersManager::save()
 	QSettings history(SessionsManager::getWritableDataPath(QLatin1String("transfers.ini")), QSettings::IniFormat);
 	history.clear();
 
-	const int limit(SettingsManager::getValue(SettingsManager::History_DownloadsLimitPeriodOption).toInt());
+	const int limit(SettingsManager::getOption(SettingsManager::History_DownloadsLimitPeriodOption).toInt());
 	int entry(1);
 
 	for (int i = 0; i < m_transfers.count(); ++i)
 	{
-		if (m_privateTransfers.contains(m_transfers.at(i)) || (m_transfers.at(i)->getState() == Transfer::FinishedState && m_transfers.at(i)->getTimeFinished().isValid() && m_transfers.at(i)->getTimeFinished().daysTo(QDateTime::currentDateTime()) > limit))
+		if (m_privateTransfers.contains(m_transfers.at(i)) || (m_transfers.at(i)->getState() == Transfer::FinishedState && m_transfers.at(i)->getTimeFinished().isValid() && m_transfers.at(i)->getTimeFinished().daysTo(QDateTime::currentDateTimeUtc()) > limit))
 		{
 			continue;
 		}
@@ -970,7 +1073,7 @@ void TransfersManager::save()
 		history.setValue(QStringLiteral("%1/source").arg(entry), m_transfers.at(i)->getSource().toString());
 		history.setValue(QStringLiteral("%1/target").arg(entry), m_transfers.at(i)->getTarget());
 		history.setValue(QStringLiteral("%1/timeStarted").arg(entry), m_transfers.at(i)->getTimeStarted().toString(Qt::ISODate));
-		history.setValue(QStringLiteral("%1/timeFinished").arg(entry), ((m_transfers.at(i)->getTimeFinished().isValid() && m_transfers.at(i)->getState() != Transfer::RunningState) ? m_transfers.at(i)->getTimeFinished() : QDateTime::currentDateTime()).toString(Qt::ISODate));
+		history.setValue(QStringLiteral("%1/timeFinished").arg(entry), ((m_transfers.at(i)->getTimeFinished().isValid() && m_transfers.at(i)->getState() != Transfer::RunningState) ? m_transfers.at(i)->getTimeFinished() : QDateTime::currentDateTimeUtc()).toString(Qt::ISODate));
 		history.setValue(QStringLiteral("%1/bytesTotal").arg(entry), m_transfers.at(i)->getBytesTotal());
 		history.setValue(QStringLiteral("%1/bytesReceived").arg(entry), m_transfers.at(i)->getBytesReceived());
 
@@ -980,27 +1083,45 @@ void TransfersManager::save()
 	history.sync();
 }
 
-void TransfersManager::transferStarted()
+void TransfersManager::clearTransfers(int period)
+{
+	for (int i = (m_transfers.count() - 1); i >= 0; --i)
+	{
+		if (m_transfers.at(i)->getState() == Transfer::FinishedState && (period == 0 || (m_transfers.at(i)->getTimeFinished().isValid() && m_transfers.at(i)->getTimeFinished().secsTo(QDateTime::currentDateTimeUtc()) > (period * 3600))))
+		{
+			TransfersManager::removeTransfer(m_transfers.at(i));
+		}
+	}
+}
+
+void TransfersManager::handleTransferStarted()
 {
 	Transfer *transfer(qobject_cast<Transfer*>(sender()));
 
 	if (transfer && transfer->getState() != Transfer::CancelledState)
 	{
+		if (transfer->getState() == Transfer::RunningState)
+		{
+			m_hasRunningTransfers = true;
+		}
+
 		emit transferStarted(transfer);
 
 		scheduleSave();
 	}
 }
 
-void TransfersManager::transferFinished()
+void TransfersManager::handleTransferFinished()
 {
 	Transfer *transfer(qobject_cast<Transfer*>(sender()));
 
+	updateRunningTransfersState();
+
 	if (transfer)
 	{
 		if (transfer->getState() == Transfer::FinishedState)
 		{
-			connect(NotificationsManager::createNotification(NotificationsManager::TransferCompletedEvent, tr("Transfer completed:\n%1").arg(QFileInfo(transfer->getTarget()).fileName()), Notification::InformationLevel, this), SIGNAL(clicked()), transfer, SLOT(openTarget()));
+			connect(NotificationsManager::createNotification(NotificationsManager::TransferCompletedEvent, tr("Download completed:\n%1").arg(QFileInfo(transfer->getTarget()).fileName()), Notification::InformationLevel, this), &Notification::clicked, transfer, &Transfer::openTarget);
 		}
 
 		emit transferFinished(transfer);
@@ -1012,22 +1133,25 @@ void TransfersManager::transferFinished()
 	}
 }
 
-void TransfersManager::transferChanged()
+void TransfersManager::handleTransferChanged()
 {
 	Transfer *transfer(qobject_cast<Transfer*>(sender()));
 
 	if (transfer)
 	{
-		emit transferChanged(transfer);
-
 		scheduleSave();
+		updateRunningTransfersState();
+
+		emit transferChanged(transfer);
 	}
 }
 
-void TransfersManager::transferStopped()
+void TransfersManager::handleTransferStopped()
 {
 	Transfer *transfer(qobject_cast<Transfer*>(sender()));
 
+	updateRunningTransfersState();
+
 	if (transfer)
 	{
 		emit transferStopped(transfer);
@@ -1036,17 +1160,6 @@ void TransfersManager::transferStopped()
 	}
 }
 
-void TransfersManager::clearTransfers(int period)
-{
-	for (int i = (m_transfers.count() - 1); i >= 0; --i)
-	{
-		if (m_transfers.at(i)->getState() == Transfer::FinishedState && (period == 0 || (m_transfers.at(i)->getTimeFinished().isValid() && m_transfers.at(i)->getTimeFinished().secsTo(QDateTime::currentDateTime()) > (period * 3600))))
-		{
-			TransfersManager::removeTransfer(m_transfers.at(i));
-		}
-	}
-}
-
 TransfersManager* TransfersManager::getInstance()
 {
 	return m_instance;
@@ -1100,7 +1213,7 @@ Transfer* TransfersManager::startTransfer(QNetworkReply *reply, const QString &t
 	return transfer;
 }
 
-QList<Transfer*> TransfersManager::getTransfers()
+QVector<Transfer*> TransfersManager::getTransfers()
 {
 	if (!m_isInitilized)
 	{
@@ -1123,7 +1236,7 @@ QList<Transfer*> TransfersManager::getTransfers()
 
 		m_isInitilized = true;
 
-		connect(QCoreApplication::instance(), SIGNAL(aboutToQuit()), m_instance, SLOT(save()));
+		connect(QCoreApplication::instance(), &Application::aboutToQuit, m_instance, &TransfersManager::save);
 	}
 
 	return m_transfers;
@@ -1190,4 +1303,9 @@ bool TransfersManager::isDownloading(const QString &source, const QString &targe
 	return false;
 }
 
+bool TransfersManager::hasRunningTransfers()
+{
+	return m_hasRunningTransfers;
+}
+
 }
