@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2015 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2021 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 * Copyright (C) 2016 - 2017 Jan Bajer aka bajasoft <jbajer@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
@@ -184,7 +184,11 @@ void QtWebEnginePage::handleLoadFinished()
 				}
 			}
 
+#if QTWEBENGINECORE_VERSION >= 0x050D00
+			const QStringList blockedRequests(m_widget->getBlockedElements());
+#else
 			const QStringList blockedRequests(qobject_cast<QtWebEngineWebBackend*>(m_widget->getBackend())->getBlockedElements(url.host()));
+#endif
 
 			if (!blockedRequests.isEmpty())
 			{
@@ -271,7 +275,14 @@ void QtWebEnginePage::setHistory(const WindowHistoryInformation &history)
 	stream.device()->reset();
 	stream >> *(this->history());
 
-	this->history()->goToItem(this->history()->itemAt(history.index));
+	if (this->history()->count() == history.entries.count() && !Utils::isUrlEmpty(this->history()->currentItem().url()))
+	{
+		this->history()->goToItem(this->history()->itemAt(history.index));
+	}
+	else
+	{
+		setUrl(history.entries.value(history.index).url);
+	}
 }
 
 QtWebEngineWebWidget* QtWebEnginePage::getWebWidget() const
@@ -374,7 +385,7 @@ QVariant QtWebEnginePage::runScriptSource(const QString &script)
 	QVariant result;
 	QEventLoop eventLoop;
 
-	runJavaScript(script, [&](const QVariant &runResult)
+	runJavaScript(script, QWebEngineScript::UserWorld, [&](const QVariant &runResult)
 	{
 		result = runResult;
 
