@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2015 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2019 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 * Copyright (C) 2016 Jan Bajer aka bajasoft <jbajer@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
@@ -23,6 +23,7 @@
 #include "QtWebEngineTransfer.h"
 #include "QtWebEngineUrlRequestInterceptor.h"
 #include "QtWebEngineWebWidget.h"
+#include "../../../../core/ContentFiltersManager.h"
 #include "../../../../core/HandlersManager.h"
 #include "../../../../core/NetworkManagerFactory.h"
 #include "../../../../core/SettingsManager.h"
@@ -44,7 +45,9 @@ QHash<QString, QString> QtWebEngineWebBackend::m_userAgentComponents;
 QMap<QString, QString> QtWebEngineWebBackend::m_userAgents;
 
 QtWebEngineWebBackend::QtWebEngineWebBackend(QObject *parent) : WebBackend(parent),
+#if QTWEBENGINECORE_VERSION < 0x050D00
 	m_requestInterceptor(nullptr),
+#endif
 	m_isInitialized(false)
 {
 	const QString userAgent(QWebEngineProfile::defaultProfile()->httpUserAgent());
@@ -190,11 +193,16 @@ WebWidget* QtWebEngineWebBackend::createWidget(const QVariantMap &parameters, Co
 	if (!m_isInitialized)
 	{
 		m_isInitialized = true;
+
+		ContentFiltersManager::initialize();
+
+#if QTWEBENGINECORE_VERSION < 0x050D00
 		m_requestInterceptor = new QtWebEngineUrlRequestInterceptor(this);
 
+		QWebEngineProfile::defaultProfile()->setRequestInterceptor(m_requestInterceptor);
+#endif
 		QWebEngineProfile::defaultProfile()->setHttpAcceptLanguage(NetworkManagerFactory::getAcceptLanguage());
 		QWebEngineProfile::defaultProfile()->setHttpUserAgent(getUserAgent());
-		QWebEngineProfile::defaultProfile()->setRequestInterceptor(m_requestInterceptor);
 
 		QWebEngineSettings::globalSettings()->setAttribute(QWebEngineSettings::FullScreenSupportEnabled, true);
 		QWebEngineSettings::globalSettings()->setAttribute(QWebEngineSettings::FocusOnNavigationEnabled, false);
@@ -279,10 +287,12 @@ QString QtWebEngineWebBackend::getUserAgent(const QString &pattern) const
 	return ((userAgent.value.isEmpty()) ? QString() : getUserAgent(userAgent.value));
 }
 
+#if QTWEBENGINECORE_VERSION < 0x050D00
 QStringList QtWebEngineWebBackend::getBlockedElements(const QString &domain) const
 {
 	return (m_requestInterceptor ? m_requestInterceptor->getBlockedElements(domain) : QStringList());
 }
+#endif
 
 QUrl QtWebEngineWebBackend::getHomePage() const
 {
