@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2015 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2021 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,9 @@
 
 #include "QtWebEngineWebWidget.h"
 #include "QtWebEnginePage.h"
+#if QTWEBENGINECORE_VERSION >= 0x050D00
+#include "QtWebEngineUrlRequestInterceptor.h"
+#endif
 #include "../../../../core/Application.h"
 #include "../../../../core/BookmarksManager.h"
 #include "../../../../core/Console.h"
@@ -62,23 +65,49 @@
 namespace Otter
 {
 
+#if QTWEBENGINECORE_VERSION >= 0x050B00
+QtWebEngineInspectorWidget::QtWebEngineInspectorWidget(QWebEnginePage *page, QWidget *parent) : QWebEngineView(parent),
+	m_page(page)
+{
+	setMinimumHeight(200);
+}
+
+void QtWebEngineInspectorWidget::showEvent(QShowEvent *event)
+{
+	if (!page()->inspectedPage())
+	{
+		page()->setInspectedPage(m_page);
+	}
+
+	QWebEngineView::showEvent(event);
+}
+#endif
+
 QtWebEngineWebWidget::QtWebEngineWebWidget(const QVariantMap &parameters, WebBackend *backend, ContentsWidget *parent) : WebWidget(parameters, backend, parent),
 	m_webView(nullptr),
 #if QTWEBENGINECORE_VERSION >= 0x050B00
-	m_inspectorView(nullptr),
+	m_inspectorWidget(nullptr),
 #endif
 	m_page(new QtWebEnginePage(SessionsManager::calculateOpenHints(parameters).testFlag(SessionsManager::PrivateOpen), this)),
+#if QTWEBENGINECORE_VERSION >= 0x050D00
+	m_requestInterceptor(new QtWebEngineUrlRequestInterceptor(this)),
+#endif
 	m_loadingState(FinishedLoadingState),
 	m_canGoForwardValue(UnknownValue),
 	m_documentLoadingProgress(0),
 	m_focusProxyTimer(0),
 	m_updateNavigationActionsTimer(0),
+	m_isClosing(false),
 	m_isEditing(false),
 	m_isFullScreen(false),
 	m_isTypedIn(false)
 {
 	setFocusPolicy(Qt::StrongFocus);
 
+#if QTWEBENGINECORE_VERSION >= 0x050D00
+	m_page->setUrlRequestInterceptor(m_requestInterceptor);
+#endif
+
 	connect(m_page, &QtWebEnginePage::loadProgress, [&](int progress)
 	{
 		m_documentLoadingProgress = progress;
@@ -114,6 +143,14 @@ QtWebEngineWebWidget::QtWebEngineWebWidget(const QVariantMap &parameters, WebBac
 	connect(m_page, &QtWebEnginePage::renderProcessTerminated, this, &QtWebEngineWebWidget::notifyRenderProcessTerminated);
 	connect(m_page->action(QWebEnginePage::Redo), &QAction::changed, this, &QtWebEngineWebWidget::notifyRedoActionStateChanged);
 	connect(m_page->action(QWebEnginePage::Undo), &QAction::changed, this, &QtWebEngineWebWidget::notifyUndoActionStateChanged);
+#if QTWEBENGINECORE_VERSION >= 0x050D00
+	connect(m_requestInterceptor, &QtWebEngineUrlRequestInterceptor::requestBlocked, this, &QtWebEngineWebWidget::requestBlocked);
+#endif
+}
+
+QtWebEngineWebWidget::~QtWebEngineWebWidget()
+{
+	m_isClosing = true;
 }
 
 void QtWebEngineWebWidget::timerEvent(QTimerEvent *event)
@@ -896,7 +933,7 @@ void QtWebEngineWebWidget::triggerAction(int identifier, const QVariantMap &para
 			{
 				const bool showInspector(parameters.value(QLatin1String("isChecked"), !getActionState(identifier, parameters).isChecked).toBool());
 
-				if (showInspector && !m_inspectorView)
+				if (showInspector && !m_inspectorWidget)
 				{
 					getInspector();
 				}
@@ -983,22 +1020,25 @@ void QtWebEngineWebWidget::handleLoadFinished()
 	notifyNavigationActionsChanged();
 	startReloadTimer();
 
-	m_page->runJavaScript(getFastForwardScript(false), [&](const QVariant &result)
+	QTimer::singleShot(100, this, [&]()
 	{
-		m_canGoForwardValue = (result.toBool() ? TrueValue : FalseValue);
-
-		emit arbitraryActionsStateChanged({ActionsManager::FastForwardAction});
-	});
-
-	const QVector<ChangeWatcher> watchers({FeedsWatcher, LinksWatcher, MetaDataWatcher, SearchEnginesWatcher, StylesheetsWatcher});
-
-	for (int i = 0; i < watchers.count(); ++i)
-	{
-		if (isWatchingChanges(watchers.at(i)))
+		m_page->runJavaScript(getFastForwardScript(false), [&](const QVariant &result)
 		{
-			updateWatchedData(watchers.at(i));
+			m_canGoForwardValue = (result.toBool() ? TrueValue : FalseValue);
+
+			emit arbitraryActionsStateChanged({ActionsManager::FastForwardAction});
+		});
+
+		const QVector<ChangeWatcher> watchers({FeedsWatcher, LinksWatcher, MetaDataWatcher, SearchEnginesWatcher, StylesheetsWatcher});
+
+		for (int i = 0; i < watchers.count(); ++i)
+		{
+			if (isWatchingChanges(watchers.at(i)))
+			{
+				updateWatchedData(watchers.at(i));
+			}
 		}
-	}
+	});
 
 	emit contentStateChanged(getContentState());
 	emit loadingStateChanged(FinishedLoadingState);
@@ -1218,6 +1258,11 @@ void QtWebEngineWebWidget::notifyNavigationActionsChanged()
 
 void QtWebEngineWebWidget::notifyWatchedDataChanged(ChangeWatcher watcher)
 {
+	if (m_isClosing)
+	{
+		return;
+	}
+
 	if (watcher >= m_watchedChanges.count())
 	{
 		m_watchedChanges.resize(watcher + 1);
@@ -1252,6 +1297,10 @@ void QtWebEngineWebWidget::updateOptions(const QUrl &url)
 	{
 		connect(m_page, &QtWebEnginePage::geometryChangeRequested, this, &QtWebEngineWebWidget::requestedGeometryChange);
 	}
+
+#if QTWEBENGINECORE_VERSION >= 0x050D00
+	m_requestInterceptor->updateOptions(url);
+#endif
 }
 
 void QtWebEngineWebWidget::updateWatchedData(ChangeWatcher watcher)
@@ -1501,18 +1550,12 @@ WebWidget* QtWebEngineWebWidget::clone(bool cloneHistory, bool isPrivate, const
 #if QTWEBENGINECORE_VERSION >= 0x050B00
 QWidget* QtWebEngineWebWidget::getInspector()
 {
-	if (!m_inspectorView)
+	if (!m_inspectorWidget)
 	{
-		m_inspectorView = new QWebEngineView(this);
-		m_inspectorView->setMinimumHeight(200);
-
-		QTimer::singleShot(100, this, [&]()
-		{
-			m_inspectorView->page()->setInspectedPage(m_page);
-		});
+		m_inspectorWidget = new QtWebEngineInspectorWidget(m_page, this);
 	}
 
-	return m_inspectorView;
+	return m_inspectorWidget;
 }
 #endif
 
@@ -1609,6 +1652,13 @@ QDateTime QtWebEngineWebWidget::getLastUrlClickTime() const
 	return m_lastUrlClickTime;
 }
 
+#if QTWEBENGINECORE_VERSION >= 0x050D00
+QStringList QtWebEngineWebWidget::getBlockedElements() const
+{
+	return m_requestInterceptor->getBlockedElements();
+}
+#endif
+
 QPoint QtWebEngineWebWidget::getScrollPosition() const
 {
 	return m_page->scrollPosition().toPoint();
@@ -1724,6 +1774,13 @@ QVector<WebWidget::LinkUrl> QtWebEngineWebWidget::getSearchEngines() const
 	return m_searchEngines;
 }
 
+#if QTWEBENGINECORE_VERSION >= 0x050D00
+QVector<NetworkManager::ResourceInformation> QtWebEngineWebWidget::getBlockedRequests() const
+{
+	return m_requestInterceptor->getBlockedRequests();
+}
+#endif
+
 QMultiMap<QString, QString> QtWebEngineWebWidget::getMetaData() const
 {
 	return m_metaData;
@@ -1850,7 +1907,7 @@ bool QtWebEngineWebWidget::isFullScreen() const
 #if QTWEBENGINECORE_VERSION >= 0x050B00
 bool QtWebEngineWebWidget::isInspecting() const
 {
-	return (m_inspectorView && m_inspectorView->isVisible());
+	return (m_inspectorWidget && m_inspectorWidget->isVisible());
 }
 #endif
 
