@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2015 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,11 +9,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -21,14 +21,13 @@
 #define OTTER_QTWEBKITINSPECTOR_H
 
 #include <QtWebKitWidgets/QWebInspector>
-#include <QtWidgets/QToolButton>
 
 namespace Otter
 {
 
 class QtWebKitWebWidget;
 
-class QtWebKitInspector : public QWebInspector
+class QtWebKitInspector final : public QWebInspector
 {
 	Q_OBJECT
 
@@ -36,20 +35,7 @@ public:
 	explicit QtWebKitInspector(QtWebKitWebWidget *parent);
 
 protected:
-	void childEvent(QChildEvent *event);
-#ifdef OTTER_ENABLE_QTWEBKIT_LEGACY
-	void showEvent(QShowEvent *event);
-	void resizeEvent(QResizeEvent *event);
-
-protected slots:
-	void hideInspector();
-#endif
-
-private:
-	QtWebKitWebWidget *m_widget;
-#ifdef OTTER_ENABLE_QTWEBKIT_LEGACY
-	QToolButton *m_closeButton;
-#endif
+	void childEvent(QChildEvent *event) override;
 };
 
 }
