@@ -1,7 +1,7 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
 * Copyright (C) 2014, 2016 Piotr Wójcik <chocimier@tlen.pl>
-* Copyright (C) 2014 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2014 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -10,18 +10,18 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
 #include "OperaSearchEnginesImporter.h"
+#include "../../../core/IniSettings.h"
 #include "../../../core/SearchEnginesManager.h"
 #include "../../../core/SessionsManager.h"
-#include "../../../core/Settings.h"
 #include "../../../core/SettingsManager.h"
 #include "../../../core/Utils.h"
 
@@ -38,20 +38,11 @@ OperaSearchEnginesImporter::OperaSearchEnginesImporter(QObject *parent) : Import
 {
 }
 
-OperaSearchEnginesImporter::~OperaSearchEnginesImporter()
-{
-	if (m_optionsWidget)
-	{
-		m_optionsWidget->deleteLater();
-	}
-}
-
-QWidget* OperaSearchEnginesImporter::getOptionsWidget()
+QWidget* OperaSearchEnginesImporter::createOptionsWidget(QWidget *parent)
 {
 	if (!m_optionsWidget)
 	{
-		m_optionsWidget = new QCheckBox(tr("Remove existing search engines"));
-		m_optionsWidget->setChecked(true);
+		m_optionsWidget = new QCheckBox(tr("Remove existing search engines"), parent);
 	}
 
 	return m_optionsWidget;
@@ -59,12 +50,12 @@ QWidget* OperaSearchEnginesImporter::getOptionsWidget()
 
 QString OperaSearchEnginesImporter::getTitle() const
 {
-	return QString(tr("Opera search engines"));
+	return tr("Opera search engines");
 }
 
 QString OperaSearchEnginesImporter::getDescription() const
 {
-	return QString(tr("Imports search engines from Opera Browser version 12 or earlier"));
+	return tr("Imports search engines from Opera Browser version 12 or earlier");
 }
 
 QString OperaSearchEnginesImporter::getVersion() const
@@ -74,10 +65,16 @@ QString OperaSearchEnginesImporter::getVersion() const
 
 QString OperaSearchEnginesImporter::getSuggestedPath(const QString &path) const
 {
-	if (!path.isEmpty() && QFileInfo(path).isDir())
+	if (!path.isEmpty())
 	{
-		return QDir(path).filePath(QLatin1String("search.ini"));
+		if (QFileInfo(path).isDir())
+		{
+			return QDir(path).filePath(QLatin1String("search.ini"));
+		}
+
+		return path;
 	}
+
 #if !defined(Q_OS_MAC) && defined(Q_OS_UNIX)
 	const QString homePath(QStandardPaths::standardLocations(QStandardPaths::HomeLocation).value(0));
 
@@ -97,69 +94,65 @@ QString OperaSearchEnginesImporter::getBrowser() const
 
 QUrl OperaSearchEnginesImporter::getHomePage() const
 {
-	return QUrl(QLatin1String("http://otter-browser.org/"));
-}
-
-QIcon OperaSearchEnginesImporter::getIcon() const
-{
-	return QIcon();
+	return QUrl(QLatin1String("https://otter-browser.org/"));
 }
 
 QStringList OperaSearchEnginesImporter::getFileFilters() const
 {
-	return QStringList(tr("Opera search engines files (search.ini)"));
+	return {tr("Opera search engines files (search.ini)")};
 }
 
-ImportType OperaSearchEnginesImporter::getImportType() const
+Importer::ImportType OperaSearchEnginesImporter::getImportType() const
 {
 	return SearchEnginesImport;
 }
 
 bool OperaSearchEnginesImporter::import(const QString &path)
 {
-	Settings settings(getSuggestedPath(path), this);
+	IniSettings settings(getSuggestedPath(path), this);
+
+	if (settings.hasError())
+	{
+		emit importFinished(SearchEnginesImport, FailedImport, 0);
+
+		return false;
+	}
+
+	emit importStarted(SearchEnginesImport, -1);
 
 	if (m_optionsWidget->isChecked())
 	{
-		SettingsManager::setValue(SettingsManager::Search_SearchEnginesOrderOption, QStringList());
+		SettingsManager::setOption(SettingsManager::Search_SearchEnginesOrderOption, QStringList());
 	}
 
 	const QStringList groups(settings.getGroups());
-	QStringList identifiers;
 	QStringList keywords(SearchEnginesManager::getSearchKeywords());
 
 	settings.beginGroup(QLatin1String("Options"));
 
-	const QVariant defaultEngine(settings.getValue(QLatin1String("Default Search")));
-	const QList<QFileInfo> allSearchEngines(QDir(SessionsManager::getReadableDataPath(QLatin1String("searches"))).entryInfoList());
-
-	for (int i = 0; i < allSearchEngines.count(); ++i)
-	{
-		identifiers.append(allSearchEngines.at(i).baseName());
-	}
+	const QVariant defaultSearchEngine(settings.getValue(QLatin1String("Default Search")));
+	int totalAmount(0);
 
 	for (int i = 0; i < groups.count(); ++i)
 	{
-		if (groups.at(i).startsWith(QLatin1String("Search Engine ")))
-		{
-			settings.beginGroup(groups.at(i));
-		}
-		else
+		if (!groups.at(i).startsWith(QLatin1String("Search Engine ")))
 		{
 			continue;
 		}
 
+		settings.beginGroup(groups.at(i));
+
 		if (settings.getValue(QLatin1String("Deleted")).toInt())
 		{
 			continue;
 		}
 
 		SearchEnginesManager::SearchEngineDefinition searchEngine;
-		searchEngine.identifier = Utils::createIdentifier(settings.getValue(QLatin1String("UNIQUEID")).toString(), identifiers);
 		searchEngine.title = settings.getValue(QLatin1String("Name")).toString();
 		searchEngine.keyword = Utils::createIdentifier(settings.getValue(QLatin1String("Key")).toString(), keywords);
 		searchEngine.encoding = settings.getValue(QLatin1String("Encoding")).toString();
 		searchEngine.resultsUrl.url = settings.getValue(QLatin1String("URL")).toString().replace(QLatin1String("%s"), QLatin1String("{searchTerms}"));
+		searchEngine.identifier = searchEngine.createIdentifier();
 
 		if (settings.getValue(QLatin1String("Is post")).toInt())
 		{
@@ -178,12 +171,22 @@ bool OperaSearchEnginesImporter::import(const QString &path)
 			searchEngine.suggestionsUrl.method = QLatin1String("get");
 		}
 
-		SearchEnginesManager::addSearchEngine(searchEngine, (settings.getValue(QLatin1String("UNIQUEID")) == defaultEngine));
+		if (SearchEnginesManager::addSearchEngine(searchEngine))
+		{
+			if (m_optionsWidget->isChecked() && settings.getValue(QLatin1String("UNIQUEID")) == defaultSearchEngine)
+			{
+				SettingsManager::setOption(SettingsManager::Search_DefaultSearchEngineOption, searchEngine.identifier);
+				SettingsManager::setOption(SettingsManager::Search_DefaultQuickSearchEngineOption, searchEngine.identifier);
+			}
 
-		identifiers.append(searchEngine.identifier);
-		keywords.append(searchEngine.keyword);
+			keywords.append(searchEngine.keyword);
+
+			++totalAmount;
+		}
 	}
 
+	emit importFinished(SearchEnginesImport, SuccessfullImport, totalAmount);
+
 	return true;
 }
 
