@@ -1,7 +1,7 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
 * Copyright (C) 2014 - 2015 Piotr Wójcik <chocimier@tlen.pl>
-* Copyright (C) 2014 - 2015 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2014 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -10,30 +10,33 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
 #include "OperaSessionImporter.h"
+#include "../../../core/IniSettings.h"
 #include "../../../core/SessionsManager.h"
-#include "../../../core/Settings.h"
 
 #include <QtCore/QDir>
+#include <QtCore/QStandardPaths>
 
 namespace Otter
 {
 
-OperaSessionImporter::OperaSessionImporter(QObject *parent): Importer(parent)
+OperaSessionImporter::OperaSessionImporter(QObject *parent) : Importer(parent)
 {
 }
 
-QWidget* OperaSessionImporter::getOptionsWidget()
+QWidget* OperaSessionImporter::createOptionsWidget(QWidget *parent)
 {
-	return NULL;
+	Q_UNUSED(parent)
+
+	return nullptr;
 }
 
 QString OperaSessionImporter::getTitle() const
@@ -51,17 +54,25 @@ QString OperaSessionImporter::getVersion() const
 	return QLatin1String("0.2");
 }
 
-QString OperaSessionImporter::getFileFilter() const
-{
-	return tr("Opera session files (*.win)");
-}
-
 QString OperaSessionImporter::getSuggestedPath(const QString &path) const
 {
-	if (!path.isEmpty() && QFileInfo(path).isDir())
+	if (!path.isEmpty())
 	{
-		return QDir(path).filePath(QLatin1String("sessions/autosave.win"));
+		if (QFileInfo(path).isDir())
+		{
+			return QDir(path).filePath(QLatin1String("sessions/autosave.win"));
+		}
+
+		return path;
 	}
+#if !defined(Q_OS_MAC) && defined(Q_OS_UNIX)
+	const QString homePath(QStandardPaths::standardLocations(QStandardPaths::HomeLocation).value(0));
+
+	if (!homePath.isEmpty())
+	{
+		return QDir(homePath).filePath(QLatin1String(".opera/sessions/autosave.win"));
+	}
+#endif
 
 	return path;
 }
@@ -73,15 +84,15 @@ QString OperaSessionImporter::getBrowser() const
 
 QUrl OperaSessionImporter::getHomePage() const
 {
-	return QUrl(QLatin1String("http://otter-browser.org/"));
+	return QUrl(QLatin1String("https://otter-browser.org/"));
 }
 
-QIcon OperaSessionImporter::getIcon() const
+QStringList OperaSessionImporter::getFileFilters() const
 {
-	return QIcon();
+	return {tr("Opera session files (*.win)")};
 }
 
-ImportType OperaSessionImporter::getType() const
+Importer::ImportType OperaSessionImporter::getImportType() const
 {
 	return SessionsImport;
 }
@@ -89,18 +100,22 @@ ImportType OperaSessionImporter::getType() const
 bool OperaSessionImporter::import(const QString &path)
 {
 	QHash<int, SessionMainWindow*> mainWindows;
-	Settings originalSession(getSuggestedPath(path));
+	IniSettings originalSession(getSuggestedPath(path));
 	originalSession.beginGroup(QLatin1String("session"));
 
 	if (originalSession.getValue(QLatin1String("version")).toInt() == 0)
 	{
+		emit importFinished(SessionsImport, FailedImport, 0);
+
 		return false;
 	}
 
+	emit importStarted(SessionsImport, 1);
+
 	SessionInformation session;
 	session.title = QFileInfo(path).completeBaseName();
 
-	const int windowCount = originalSession.getValue(QLatin1String("window count")).toInt();
+	const int windowCount(originalSession.getValue(QLatin1String("window count")).toInt());
 
 	for (int i = 1; i <= windowCount; ++i)
 	{
@@ -108,7 +123,7 @@ bool OperaSessionImporter::import(const QString &path)
 
 		if (originalSession.getValue(QLatin1String("type")).toInt() == 0)
 		{
-			SessionMainWindow *mainWindow = new SessionMainWindow();
+			SessionMainWindow *mainWindow(new SessionMainWindow());
 
 			mainWindows.insert(originalSession.getValue(QLatin1String("id")).toInt(), mainWindow);
 
@@ -116,40 +131,39 @@ bool OperaSessionImporter::import(const QString &path)
 		}
 
 		SessionWindow window;
-		window.geometry.setX(originalSession.getValue(QLatin1String("x")).toInt());
-		window.geometry.setY(originalSession.getValue(QLatin1String("y")).toInt());
-		window.geometry.setWidth(originalSession.getValue(QLatin1String("w")).toInt());
-		window.geometry.setHeight(originalSession.getValue(QLatin1String("h")).toInt());
-
-		if (originalSession.getValue(QLatin1String("user auto reload enable")).toInt() == 1)
-		{
-			window.overrides[QLatin1String("Content/PageReloadTime")] = originalSession.getValue(QLatin1String("user auto reload sec user setting")).toInt();
-		}
+		WindowState windowState;
+		windowState.geometry = QRect(originalSession.getValue(QLatin1String("x")).toInt(), originalSession.getValue(QLatin1String("y")).toInt(), originalSession.getValue(QLatin1String("w")).toInt(), originalSession.getValue(QLatin1String("h")).toInt());
 
 		switch (originalSession.getValue(QLatin1String("state")).toInt())
 		{
 			case 0:
-				window.state = NormalWindowState;
+				windowState.state = Qt::WindowNoState;
 
 				break;
 			case 1:
-				window.state = MinimizedWindowState;
+				windowState.state = Qt::WindowMinimized;
 
 				break;
 			default:
-				window.state = MaximizedWindowState;
+				windowState.state = Qt::WindowMaximized;
 
 				break;
 		}
 
+		window.state = windowState;
 		window.historyIndex = (originalSession.getValue(QLatin1String("current history")).toInt() - 1);
 		window.isPinned = originalSession.getValue(QLatin1String("locked")).toInt();
 
-		const int zoom = originalSession.getValue(QLatin1String("scale")).toInt();
+		if (originalSession.getValue(QLatin1String("user auto reload enable")).toInt() == 1)
+		{
+			window.options[SettingsManager::Content_PageReloadTimeOption] = originalSession.getValue(QLatin1String("user auto reload sec user setting")).toInt();
+		}
+
+		const int zoom(originalSession.getValue(QLatin1String("scale")).toInt());
 
 		originalSession.beginGroup(QString::number(i) + QLatin1String("history url"));
 
-		const int historyCount = originalSession.getValue(QLatin1String("count")).toInt();
+		const int historyCount(originalSession.getValue(QLatin1String("count")).toInt());
 
 		for (int j = 0; j < historyCount; ++j)
 		{
@@ -179,7 +193,7 @@ bool OperaSessionImporter::import(const QString &path)
 		{
 			originalSession.beginGroup(QString::number(i));
 
-			const QString panel = originalSession.getValue(QLatin1String("panel type")).toString();
+			const QString panel(originalSession.getValue(QLatin1String("panel type")).toString());
 
 			if (!panel.isEmpty())
 			{
@@ -221,7 +235,7 @@ bool OperaSessionImporter::import(const QString &path)
 			window.historyIndex = (window.historyIndex + 1);
 		}
 
-		const int parent = originalSession.getValue(QLatin1String("parent")).toInt();
+		const int parent(originalSession.getValue(QLatin1String("parent")).toInt());
 
 		if (parent > 0 && mainWindows.contains(parent))
 		{
@@ -241,10 +255,12 @@ bool OperaSessionImporter::import(const QString &path)
 		session.windows.append(**iterator);
 	}
 
-	const bool result = SessionsManager::saveSession(session);
+	const bool result(SessionsManager::saveSession(session));
 
 	qDeleteAll(mainWindows);
 
+	emit importFinished(SessionsImport, (result ? SuccessfullImport : FailedImport), 1);
+
 	return result;
 }
 
