@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,11 +9,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -25,14 +25,14 @@
 namespace Otter
 {
 
-class MainWindow;
+class Window;
 
-class ZoomWidget : public QSlider
+class ZoomWidget final : public QSlider
 {
 	Q_OBJECT
 
 public:
-	explicit ZoomWidget(QWidget *parent = nullptr);
+	explicit ZoomWidget(Window *window, QWidget *parent = nullptr);
 
 	QSize sizeHint() const override;
 
@@ -40,10 +40,14 @@ public slots:
 	void setZoom(int zoom);
 
 protected:
+	void changeEvent(QEvent *event) override;
 	void mousePressEvent(QMouseEvent *event) override;
 
+protected slots:
+	void setWindow(Window *window);
+
 private:
-	MainWindow *m_mainWindow;
+	Window *m_window;
 };
 
 }
