@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 * Copyright (C) 2015 Piotr Wójcik <chocimier@tlen.pl>
 *
 * This program is free software: you can redistribute it and/or modify
@@ -10,46 +10,43 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
 #ifndef OTTER_BOOKMARKSCOMBOBOXWIDGET_H
 #define OTTER_BOOKMARKSCOMBOBOXWIDGET_H
 
+#include "ComboBoxWidget.h"
 #include "../core/BookmarksModel.h"
 
-#include <QtGui/QStandardItem>
-#include <QtWidgets/QComboBox>
-#include <QtWidgets/QTreeView>
-
 namespace Otter
 {
 
-class BookmarksItem;
-
-class BookmarksComboBoxWidget : public QComboBox
+class BookmarksComboBoxWidget final : public ComboBoxWidget
 {
 	Q_OBJECT
 
 public:
 	explicit BookmarksComboBoxWidget(QWidget *parent = nullptr);
 
-	void setCurrentFolder(BookmarksItem *folder);
+	void setCurrentFolder(BookmarksModel::Bookmark *folder);
 	void setMode(BookmarksModel::FormatMode mode);
-	BookmarksItem* getCurrentFolder();
+	BookmarksModel::Bookmark* getCurrentFolder() const;
+
+public slots:
+	void createFolder();
 
 protected slots:
-	void createFolder();
-	void updateBranch(QStandardItem *branch = nullptr);
+	void handleLayoutChanged();
+	void updateBranch(const QModelIndex &parent = {});
 
 private:
-	QTreeView *m_view;
-	BookmarksModel::FormatMode m_mode;
+	BookmarksModel *m_model;
 };
 
 }
