@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,11 +9,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -27,16 +27,16 @@
 namespace Otter
 {
 
-class PreviewWidget : public QFrame
+class PreviewWidget final : public QFrame
 {
 	Q_OBJECT
 
 public:
-	explicit PreviewWidget(QWidget *parent = 0);
+	explicit PreviewWidget(QWidget *parent = nullptr);
 
 public slots:
 	void setPosition(const QPoint &position);
-	void setPreview(const QString &text, const QPixmap &pixmap = QPixmap(), bool showFullText = false);
+	void setPreview(const QString &text, const QPixmap &pixmap = {}, bool showFullText = false);
 
 private:
 	QLabel *m_textLabel;
