@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2015 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,15 +9,16 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
 #include "SourceViewerWebWidget.h"
+#include "Menu.h"
 #include "SourceViewerWidget.h"
 #include "../core/Console.h"
 #include "../core/HistoryManager.h"
@@ -31,12 +32,13 @@
 #include <QtGui/QClipboard>
 #include <QtWidgets/QApplication>
 #include <QtWidgets/QMessageBox>
+#include <QtWidgets/QScrollBar>
 #include <QtWidgets/QVBoxLayout>
 
 namespace Otter
 {
 
-SourceViewerWebWidget::SourceViewerWebWidget(bool isPrivate, ContentsWidget *parent) : WebWidget(isPrivate, nullptr, parent),
+SourceViewerWebWidget::SourceViewerWebWidget(bool isPrivate, ContentsWidget *parent) : WebWidget({}, nullptr, parent),
 	m_sourceViewer(new SourceViewerWidget(this)),
 	m_networkManager(nullptr),
 	m_viewSourceReply(nullptr),
@@ -52,24 +54,22 @@ SourceViewerWebWidget::SourceViewerWebWidget(bool isPrivate, ContentsWidget *par
 
 	setContextMenuPolicy(Qt::CustomContextMenu);
 
-	connect(this, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(showContextMenu(QPoint)));
-	connect(m_sourceViewer, SIGNAL(zoomChanged(int)), this, SIGNAL(zoomChanged(int)));
-	connect(m_sourceViewer, SIGNAL(zoomChanged(int)), this, SLOT(handleZoomChange()));
-	connect(m_sourceViewer, SIGNAL(copyAvailable(bool)), this, SLOT(updateEditActions()));
+	connect(this, &SourceViewerWebWidget::customContextMenuRequested, this, &SourceViewerWebWidget::showContextMenu);
+	connect(m_sourceViewer, &SourceViewerWidget::zoomChanged, this, &SourceViewerWebWidget::zoomChanged);
+	connect(m_sourceViewer, &SourceViewerWidget::zoomChanged, this, &SourceViewerWebWidget::handleZoomChanged);
+	connect(m_sourceViewer, &SourceViewerWidget::redoAvailable, this, &SourceViewerWebWidget::notifyRedoActionStateChanged);
+	connect(m_sourceViewer, &SourceViewerWidget::undoAvailable, this, &SourceViewerWebWidget::notifyUndoActionStateChanged);
+	connect(m_sourceViewer, &SourceViewerWidget::copyAvailable, this, &SourceViewerWebWidget::notifyEditingActionsStateChanged);
+	connect(m_sourceViewer, &SourceViewerWidget::cursorPositionChanged, this, &SourceViewerWebWidget::notifyEditingActionsStateChanged);
 }
 
-void SourceViewerWebWidget::triggerAction(int identifier, const QVariantMap &parameters)
+void SourceViewerWebWidget::triggerAction(int identifier, const QVariantMap &parameters, ActionsManager::TriggerType trigger)
 {
-	if (parameters.contains(QLatin1String("isBounced")))
-	{
-		return;
-	}
-
 	switch (identifier)
 	{
 		case ActionsManager::SaveAction:
 			{
-				const QString path(Utils::getSavePath(suggestSaveFileName(SingleHtmlFileSaveFormat)).path);
+				const QString path(Utils::getSavePath(suggestSaveFileName(SingleFileSaveFormat)).path);
 
 				if (!path.isEmpty())
 				{
@@ -91,11 +91,11 @@ void SourceViewerWebWidget::triggerAction(int identifier, const QVariantMap &par
 				}
 			}
 
-			return;
+			break;
 		case ActionsManager::StopAction:
 			if (m_viewSourceReply)
 			{
-				disconnect(m_viewSourceReply, SIGNAL(finished()), this, SLOT(viewSourceReplyFinished()));
+				disconnect(m_viewSourceReply, &QNetworkReply::finished, this, &SourceViewerWebWidget::handleViewSourceReplyFinished);
 
 				m_viewSourceReply->abort();
 				m_viewSourceReply->deleteLater();
@@ -104,11 +104,10 @@ void SourceViewerWebWidget::triggerAction(int identifier, const QVariantMap &par
 
 			m_isLoading = false;
 
-			emit loadingStateChanged(WindowsManager::FinishedLoadingState);
+			emit categorizedActionsStateChanged({ActionsManager::ActionDefinition::NavigationCategory});
+			emit loadingStateChanged(WebWidget::FinishedLoadingState);
 
-			updateNavigationActions();
-
-			return;
+			break;
 		case ActionsManager::ReloadAction:
 			{
 				if (m_sourceViewer->document()->isModified())
@@ -117,19 +116,20 @@ void SourceViewerWebWidget::triggerAction(int identifier, const QVariantMap &par
 
 					if (result == QMessageBox::Cancel)
 					{
-						return;
+						break;
 					}
 
 					if (result == QMessageBox::Save)
 					{
-						triggerAction(ActionsManager::SaveAction);
+						triggerAction(ActionsManager::SaveAction, {}, trigger);
 					}
 				}
 
-				triggerAction(ActionsManager::StopAction);
+				triggerAction(ActionsManager::StopAction, {}, trigger);
 
 				QNetworkRequest request(QUrl(getUrl().toString().mid(12)));
 				request.setAttribute(QNetworkRequest::CacheLoadControlAttribute, QNetworkRequest::PreferCache);
+				request.setHeader(QNetworkRequest::UserAgentHeader, NetworkManagerFactory::getUserAgent());
 
 				if (!m_networkManager)
 				{
@@ -138,110 +138,123 @@ void SourceViewerWebWidget::triggerAction(int identifier, const QVariantMap &par
 
 				m_viewSourceReply = m_networkManager->get(request);
 
-				connect(m_viewSourceReply, SIGNAL(finished()), this, SLOT(viewSourceReplyFinished()));
+				connect(m_viewSourceReply, &QNetworkReply::finished, this, &SourceViewerWebWidget::handleViewSourceReplyFinished);
 
 				m_isLoading = true;
 
-				emit loadingStateChanged(WindowsManager::OngoingLoadingState);
-
-				updateNavigationActions();
+				emit categorizedActionsStateChanged({ActionsManager::ActionDefinition::NavigationCategory});
+				emit loadingStateChanged(WebWidget::OngoingLoadingState);
 			}
 
-			return;
+			break;
 		case ActionsManager::ReloadOrStopAction:
 			if (m_isLoading)
 			{
-				triggerAction(ActionsManager::StopAction);
+				triggerAction(ActionsManager::StopAction, {}, trigger);
 			}
 			else
 			{
-				triggerAction(ActionsManager::ReloadAction);
+				triggerAction(ActionsManager::ReloadAction, {}, trigger);
 			}
 
-			return;
+			break;
 		case ActionsManager::ReloadAndBypassCacheAction:
 			{
-				triggerAction(ActionsManager::StopAction);
+				triggerAction(ActionsManager::StopAction, {}, trigger);
 
 				QNetworkRequest request(QUrl(getUrl().toString().mid(12)));
 				request.setAttribute(QNetworkRequest::CacheLoadControlAttribute, QNetworkRequest::AlwaysNetwork);
+				request.setHeader(QNetworkRequest::UserAgentHeader, NetworkManagerFactory::getUserAgent());
 
-				m_viewSourceReply = NetworkManagerFactory::getNetworkManager()->get(request);
+				if (!m_networkManager)
+				{
+					m_networkManager = new NetworkManager(m_isPrivate, this);
+				}
 
-				connect(m_viewSourceReply, SIGNAL(finished()), this, SLOT(viewSourceReplyFinished()));
+				m_viewSourceReply = m_networkManager->get(request);
+
+				connect(m_viewSourceReply, &QNetworkReply::finished, this, &SourceViewerWebWidget::handleViewSourceReplyFinished);
 
 				m_isLoading = true;
 
-				emit loadingStateChanged(WindowsManager::OngoingLoadingState);
-
-				updateNavigationActions();
+				emit categorizedActionsStateChanged({ActionsManager::ActionDefinition::NavigationCategory});
+				emit loadingStateChanged(WebWidget::OngoingLoadingState);
 			}
 
-			return;
+			break;
 		case ActionsManager::ContextMenuAction:
 			showContextMenu();
 
-			return;
+			break;
 		case ActionsManager::UndoAction:
 			m_sourceViewer->undo();
 
-			return;
+			break;
 		case ActionsManager::RedoAction:
 			m_sourceViewer->redo();
 
-			return;
+			break;
 		case ActionsManager::CutAction:
 			m_sourceViewer->cut();
 
-			return;
+			break;
 		case ActionsManager::CopyAction:
 		case ActionsManager::CopyPlainTextAction:
 			m_sourceViewer->copy();
 
-			return;
+			break;
 		case ActionsManager::CopyAddressAction:
 			QApplication::clipboard()->setText(getUrl().toString());
 
-			return;
+			break;
 		case ActionsManager::CopyToNoteAction:
+			NotesManager::addNote(BookmarksModel::UrlBookmark, {{BookmarksModel::UrlRole, getUrl()}, {BookmarksModel::DescriptionRole, getSelectedText()}});
+
+			break;
+		case ActionsManager::PasteAction:
+			if (parameters.contains(QLatin1String("note")))
 			{
-				BookmarksItem *note(NotesManager::addNote(BookmarksModel::UrlBookmark, getUrl()));
-				note->setData(getSelectedText(), BookmarksModel::DescriptionRole);
+				const BookmarksModel::Bookmark *bookmark(NotesManager::getModel()->getBookmark(parameters[QLatin1String("note")].toULongLong()));
+
+				if (bookmark)
+				{
+					triggerAction(ActionsManager::PasteAction, {{QLatin1String("text"), bookmark->getDescription()}}, trigger);
+				}
+			}
+			else if (parameters.contains(QLatin1String("text")))
+			{
+				m_sourceViewer->textCursor().insertText(parameters[QLatin1String("text")].toString());
+			}
+			else
+			{
+				m_sourceViewer->paste();
 			}
 
-			return;
-		case ActionsManager::PasteAction:
-			m_sourceViewer->paste();
-
-			return;
+			break;
 		case ActionsManager::DeleteAction:
 			m_sourceViewer->textCursor().removeSelectedText();
 
-			return;
+			break;
 		case ActionsManager::SelectAllAction:
 			m_sourceViewer->selectAll();
 
-			return;
+			break;
 		case ActionsManager::UnselectAction:
 			m_sourceViewer->textCursor().clearSelection();
 
-			return;
+			break;
 		case ActionsManager::ClearAllAction:
 			m_sourceViewer->selectAll();
 			m_sourceViewer->textCursor().removeSelectedText();
 
-			return;
-		case ActionsManager::SearchAction:
-			quickSearch(getAction(ActionsManager::SearchAction));
-
-			return;
+			break;
 		case ActionsManager::ActivateContentAction:
 			m_sourceViewer->setFocus();
 
-			return;
+			break;
+		default:
+			break;
 	}
-
-	bounceAction(identifier, parameters);
 }
 
 void SourceViewerWebWidget::print(QPrinter *printer)
@@ -249,12 +262,7 @@ void SourceViewerWebWidget::print(QPrinter *printer)
 	m_sourceViewer->print(printer);
 }
 
-void SourceViewerWebWidget::pasteText(const QString &text)
-{
-	Q_UNUSED(text)
-}
-
-void SourceViewerWebWidget::viewSourceReplyFinished()
+void SourceViewerWebWidget::handleViewSourceReplyFinished()
 {
 	if (m_viewSourceReply)
 	{
@@ -263,62 +271,47 @@ void SourceViewerWebWidget::viewSourceReplyFinished()
 		m_viewSourceReply->deleteLater();
 		m_viewSourceReply = nullptr;
 
-		updateNavigationActions();
+		emit categorizedActionsStateChanged({ActionsManager::ActionDefinition::NavigationCategory});
 	}
 }
 
-void SourceViewerWebWidget::goToHistoryIndex(int index)
+void SourceViewerWebWidget::handleZoomChanged()
 {
-	Q_UNUSED(index)
+	SessionsManager::markSessionAsModified();
 }
 
-void SourceViewerWebWidget::removeHistoryIndex(int index, bool purge)
+void SourceViewerWebWidget::notifyEditingActionsStateChanged()
 {
-	Q_UNUSED(index)
-	Q_UNUSED(purge)
-}
-
-void SourceViewerWebWidget::handleZoomChange()
-{
-	SessionsManager::markSessionModified();
+	emit categorizedActionsStateChanged({ActionsManager::ActionDefinition::EditingCategory});
 }
 
 void SourceViewerWebWidget::showContextMenu(const QPoint &position)
 {
 	updateHitTestResult(position);
-	updateEditActions();
 
-	QWidget *child(childAt(position.isNull() ? mapFromGlobal(QCursor::pos()) : position));
-	QMenu menu;
+	emit categorizedActionsStateChanged({ActionsManager::ActionDefinition::EditingCategory});
+
+	const QWidget *child(childAt(position.isNull() ? mapFromGlobal(QCursor::pos()) : position));
+	const QPoint menuPosition(position.isNull() ? QCursor::pos() : mapToGlobal(position));
 
 	if (child && child->metaObject()->className() == QLatin1String("Otter::MarginWidget"))
 	{
-		QAction *showLineNumbersAction(menu.addAction(tr("Show Line Numbers")));
+		QMenu menu(this);
+		QAction *showLineNumbersAction(menu.addAction(tr("Show Line Numbers"), [&](bool show)
+		{
+			SettingsManager::setOption(SettingsManager::SourceViewer_ShowLineNumbersOption, show);
+		}));
 		showLineNumbersAction->setCheckable(true);
-		showLineNumbersAction->setChecked(SettingsManager::getValue(SettingsManager::SourceViewer_ShowLineNumbersOption).toBool());
+		showLineNumbersAction->setChecked(SettingsManager::getOption(SettingsManager::SourceViewer_ShowLineNumbersOption).toBool());
 
-		connect(showLineNumbersAction, SIGNAL(triggered(bool)), this, SLOT(setShowLineNumbers(bool)));
+		menu.exec(menuPosition);
 	}
 	else
 	{
-		menu.addAction(getAction(ActionsManager::UndoAction));
-		menu.addAction(getAction(ActionsManager::RedoAction));
-		menu.addSeparator();
-		menu.addAction(getAction(ActionsManager::CutAction));
-		menu.addAction(getAction(ActionsManager::CopyAction));
-		menu.addAction(getAction(ActionsManager::PasteAction));
-		menu.addAction(getAction(ActionsManager::DeleteAction));
-		menu.addSeparator();
-		menu.addAction(getAction(ActionsManager::SelectAllAction));
-		menu.addAction(getAction(ActionsManager::ClearAllAction));
+		Menu menu(Menu::UnknownMenu, this);
+		menu.load(QLatin1String("menu/webWidget.json"), {QLatin1String("edit"), QLatin1String("source")}, ActionExecutor::Object(this, this));
+		menu.exec(menuPosition);
 	}
-
-	menu.exec(position.isNull() ? QCursor::pos() : mapToGlobal(position));
-}
-
-void SourceViewerWebWidget::setShowLineNumbers(bool show)
-{
-	SettingsManager::setValue(SettingsManager::SourceViewer_ShowLineNumbersOption, show);
 }
 
 void SourceViewerWebWidget::setOption(int identifier, const QVariant &value)
@@ -333,9 +326,9 @@ void SourceViewerWebWidget::setOption(int identifier, const QVariant &value)
 	}
 }
 
-void SourceViewerWebWidget::setOptions(const QHash<int, QVariant> &options)
+void SourceViewerWebWidget::setOptions(const QHash<int, QVariant> &options, const QStringList &excludedOptions)
 {
-	WebWidget::setOptions(options);
+	WebWidget::setOptions(options, excludedOptions);
 
 	if (options.contains(SettingsManager::Content_DefaultCharacterEncodingOption))
 	{
@@ -345,7 +338,8 @@ void SourceViewerWebWidget::setOptions(const QHash<int, QVariant> &options)
 
 void SourceViewerWebWidget::setScrollPosition(const QPoint &position)
 {
-	Q_UNUSED(position)
+	m_sourceViewer->horizontalScrollBar()->setValue(position.x());
+	m_sourceViewer->verticalScrollBar()->setValue(position.y());
 }
 
 void SourceViewerWebWidget::setHistory(const WindowHistoryInformation &history)
@@ -359,19 +353,19 @@ void SourceViewerWebWidget::setZoom(int zoom)
 	{
 		m_sourceViewer->setZoom(zoom);
 
-		SessionsManager::markSessionModified();
+		SessionsManager::markSessionAsModified();
 
 		emit zoomChanged(zoom);
 	}
 }
 
-void SourceViewerWebWidget::setUrl(const QUrl &url, bool typed)
+void SourceViewerWebWidget::setUrl(const QUrl &url, bool isTyped)
 {
 	triggerAction(ActionsManager::StopAction);
 
 	m_url = url;
 
-	if (typed)
+	if (isTyped)
 	{
 		triggerAction(ActionsManager::ReloadAction);
 	}
@@ -381,7 +375,7 @@ void SourceViewerWebWidget::setContents(const QByteArray &contents, const QStrin
 {
 	triggerAction(ActionsManager::StopAction);
 
-	QTextCodec *codec(nullptr);
+	const QTextCodec *codec(nullptr);
 
 	if (hasOption(SettingsManager::Content_DefaultCharacterEncodingOption))
 	{
@@ -403,30 +397,53 @@ void SourceViewerWebWidget::setContents(const QByteArray &contents, const QStrin
 		codec = QTextCodec::codecForHtml(contents);
 	}
 
-	QString text;
-
 	if (codec)
 	{
-		text = codec->toUnicode(contents);
+		m_sourceViewer->setPlainText(codec->toUnicode(contents));
 	}
 	else
 	{
-		text = QString(contents);
+		m_sourceViewer->setPlainText(QString(contents));
 	}
 
-	m_sourceViewer->setPlainText(text);
 	m_sourceViewer->document()->setModified(false);
 }
 
-WebWidget* SourceViewerWebWidget::clone(bool cloneHistory, bool isPrivate)
+WebWidget* SourceViewerWebWidget::clone(bool cloneHistory, bool isPrivate, const QStringList &excludedOptions) const
 {
 	Q_UNUSED(cloneHistory)
 	Q_UNUSED(isPrivate)
+	Q_UNUSED(excludedOptions)
 
 	return nullptr;
 }
 
-Action* SourceViewerWebWidget::getAction(int identifier)
+QString SourceViewerWebWidget::getTitle() const
+{
+	return (m_url.isValid() ? tr("Source Viewer: %1").arg(m_url.toDisplayString().mid(12)) : tr("Source Viewer"));
+}
+
+QString SourceViewerWebWidget::getSelectedText() const
+{
+	return m_sourceViewer->textCursor().selectedText();
+}
+
+QUrl SourceViewerWebWidget::getUrl() const
+{
+	return m_url;
+}
+
+QIcon SourceViewerWebWidget::getIcon() const
+{
+	return HistoryManager::getIcon(getRequestedUrl());
+}
+
+QPoint SourceViewerWebWidget::getScrollPosition() const
+{
+	return {m_sourceViewer->horizontalScrollBar()->value(), m_sourceViewer->verticalScrollBar()->value()};
+}
+
+ActionsManager::ActionDefinition::State SourceViewerWebWidget::getActionState(int identifier, const QVariantMap &parameters) const
 {
 	switch (identifier)
 	{
@@ -451,73 +468,20 @@ Action* SourceViewerWebWidget::getAction(int identifier)
 		case ActionsManager::ZoomOutAction:
 		case ActionsManager::ZoomOriginalAction:
 		case ActionsManager::SearchAction:
-		case ActionsManager::SearchMenuAction:
 		case ActionsManager::FindAction:
 		case ActionsManager::FindNextAction:
 		case ActionsManager::FindPreviousAction:
 		case ActionsManager::QuickFindAction:
 		case ActionsManager::ActivateContentAction:
-			break;
+			return WebWidget::getActionState(identifier, parameters);
 		default:
-			return nullptr;
+			break;
 	}
 
-	if (identifier == ActionsManager::UndoAction && !getExistingAction(ActionsManager::UndoAction))
-	{
-		Action *action(WebWidget::getAction(ActionsManager::UndoAction));
-		action->setEnabled(m_sourceViewer->document()->isUndoAvailable());
+	ActionsManager::ActionDefinition::State state(ActionsManager::getActionDefinition(identifier).getDefaultState());
+	state.isEnabled = false;
 
-		connect(m_sourceViewer, SIGNAL(undoAvailable(bool)), action, SLOT(setEnabled(bool)));
-
-		return action;
-	}
-
-	if (identifier == ActionsManager::RedoAction && !getExistingAction(ActionsManager::RedoAction))
-	{
-		Action *action(WebWidget::getAction(ActionsManager::RedoAction));
-		action->setEnabled(m_sourceViewer->document()->isRedoAvailable());
-
-		connect(m_sourceViewer, SIGNAL(redoAvailable(bool)), action, SLOT(setEnabled(bool)));
-
-		return action;
-	}
-
-	return WebWidget::getAction(identifier);
-}
-
-QString SourceViewerWebWidget::getTitle() const
-{
-	return tr("Source Viewer");
-}
-
-QString SourceViewerWebWidget::getSelectedText() const
-{
-	return m_sourceViewer->textCursor().selectedText();
-}
-
-QUrl SourceViewerWebWidget::getUrl() const
-{
-	return m_url;
-}
-
-QIcon SourceViewerWebWidget::getIcon() const
-{
-	return HistoryManager::getIcon(getRequestedUrl());
-}
-
-QPixmap SourceViewerWebWidget::getThumbnail()
-{
-	return QPixmap();
-}
-
-QPoint SourceViewerWebWidget::getScrollPosition() const
-{
-	return QPoint();
-}
-
-QRect SourceViewerWebWidget::getProgressBarGeometry() const
-{
-	return QRect(QPoint(0, (height() - 30)), QSize(width(), 30));
+	return state;
 }
 
 WindowHistoryInformation SourceViewerWebWidget::getHistory() const
@@ -537,20 +501,20 @@ WindowHistoryInformation SourceViewerWebWidget::getHistory() const
 WebWidget::HitTestResult SourceViewerWebWidget::getHitTestResult(const QPoint &position)
 {
 	HitTestResult result;
-	result.position = position;
-	result.flags = IsContentEditableTest;
+	result.hitPosition = position;
+	result.flags = HitTestResult::IsContentEditableTest;
 
 	if (m_sourceViewer->document()->isEmpty())
 	{
-		result.flags |= IsEmptyTest;
+		result.flags |= HitTestResult::IsEmptyTest;
 	}
 
 	return result;
 }
 
-WindowsManager::LoadingState SourceViewerWebWidget::getLoadingState() const
+WebWidget::LoadingState SourceViewerWebWidget::getLoadingState() const
 {
-	return (m_isLoading ? WindowsManager::OngoingLoadingState : WindowsManager::FinishedLoadingState);
+	return (m_isLoading ? WebWidget::OngoingLoadingState : WebWidget::FinishedLoadingState);
 }
 
 int SourceViewerWebWidget::getZoom() const
@@ -558,6 +522,26 @@ int SourceViewerWebWidget::getZoom() const
 	return m_sourceViewer->getZoom();
 }
 
+int SourceViewerWebWidget::findInPage(const QString &text, WebWidget::FindFlags flags)
+{
+	return m_sourceViewer->findText(text, flags);
+}
+
+bool SourceViewerWebWidget::canRedo() const
+{
+	return m_sourceViewer->document()->isRedoAvailable();
+}
+
+bool SourceViewerWebWidget::canUndo() const
+{
+	return m_sourceViewer->document()->isUndoAvailable();
+}
+
+bool SourceViewerWebWidget::canViewSource() const
+{
+	return false;
+}
+
 bool SourceViewerWebWidget::hasSelection() const
 {
 	return m_sourceViewer->textCursor().hasSelection();
@@ -568,9 +552,4 @@ bool SourceViewerWebWidget::isPrivate() const
 	return m_isPrivate;
 }
 
-bool SourceViewerWebWidget::findInPage(const QString &text, WebWidget::FindFlags flags)
-{
-	return m_sourceViewer->findText(text, flags);
-}
-
 }
