@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2015 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2019 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,11 +9,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -28,7 +28,7 @@
 namespace Otter
 {
 
-class SyntaxHighlighter : public QSyntaxHighlighter
+class SyntaxHighlighter final : public QSyntaxHighlighter
 {
 	Q_OBJECT
 	Q_ENUMS(HighlightingSyntax)
@@ -38,21 +38,22 @@ public:
 	enum HighlightingSyntax
 	{
 		NoSyntax = 0,
-		HtmlSyntax = 1
+		HtmlSyntax
 	};
 
 	enum HighlightingState
 	{
 		NoState = 0,
-		DoctypeState = 1,
-		KeywordState = 2,
-		AttributeState = 3,
-		EntityState = 4,
-		ValueState = 5,
-		CommentState = 6
+		DoctypeState,
+		KeywordState,
+		AttributeState,
+		EntityState,
+		ValueState,
+		CharacterDataState,
+		CommentState
 	};
 
-	struct BlockData : public QTextBlockUserData
+	struct BlockData final : public QTextBlockUserData
 	{
 		QString context;
 		HighlightingSyntax currentSyntax = HtmlSyntax;
@@ -63,7 +64,7 @@ public:
 	explicit SyntaxHighlighter(QTextDocument *parent);
 
 protected:
-	void highlightBlock(const QString &text);
+	void highlightBlock(const QString &text) override;
 
 private:
 	static QMap<HighlightingSyntax, QMap<HighlightingState, QTextCharFormat> > m_formats;
@@ -71,30 +72,30 @@ private:
 
 class SourceViewerWidget;
 
-class MarginWidget : public QWidget
+class MarginWidget final : public QWidget
 {
 	Q_OBJECT
 
 public:
-	explicit MarginWidget(SourceViewerWidget *parent = nullptr);
+	explicit MarginWidget(SourceViewerWidget *parent);
 
 public slots:
 	void updateNumbers(const QRect &rectangle, int offset);
-	void setAmount(int amount = -1);
+	void updateWidth();
 
 protected:
-	void paintEvent(QPaintEvent *event);
-	void mousePressEvent(QMouseEvent *event);
-	void mouseMoveEvent(QMouseEvent *event);
-	void mouseReleaseEvent(QMouseEvent *event);
-	bool event(QEvent *event);
+	void paintEvent(QPaintEvent *event) override;
+	void mousePressEvent(QMouseEvent *event) override;
+	void mouseMoveEvent(QMouseEvent *event) override;
+	void mouseReleaseEvent(QMouseEvent *event) override;
+	bool event(QEvent *event) override;
 
 private:
 	SourceViewerWidget *m_sourceViewer;
 	int m_lastClickedLine;
 };
 
-class SourceViewerWidget : public QPlainTextEdit
+class SourceViewerWidget final : public QPlainTextEdit
 {
 	Q_OBJECT
 
@@ -103,15 +104,15 @@ public:
 
 	void setZoom(int zoom);
 	int getZoom() const;
-	bool findText(const QString &text, WebWidget::FindFlags flags = WebWidget::NoFlagsFind);
+	int findText(const QString &text, WebWidget::FindFlags flags = WebWidget::NoFlagsFind);
 
 protected:
-	void resizeEvent(QResizeEvent *event);
-	void focusInEvent(QFocusEvent *event);
-	void wheelEvent(QWheelEvent *event);
+	void resizeEvent(QResizeEvent *event) override;
+	void focusInEvent(QFocusEvent *event) override;
+	void wheelEvent(QWheelEvent *event) override;
 
 protected slots:
-	void optionChanged(int identifier, const QVariant &value);
+	void handleOptionChanged(int identifier, const QVariant &value);
 	void updateTextCursor();
 	void updateSelection();
 
@@ -121,6 +122,7 @@ private:
 	QTextCursor m_findTextAnchor;
 	QTextCursor m_findTextSelection;
 	WebWidget::FindFlags m_findFlags;
+	int m_findTextResultsAmount;
 	int m_zoom;
 
 signals:
