@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2015 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,18 +9,20 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
 #include "ToolButtonWidget.h"
+#include "Action.h"
+#include "MainWindow.h"
 #include "Menu.h"
 #include "ToolBarWidget.h"
-#include "../core/ActionsManager.h"
+#include "../core/Application.h"
 #include "../core/ThemesManager.h"
 
 #include <QtCore/QEvent>
@@ -31,7 +33,8 @@
 namespace Otter
 {
 
-ToolButtonWidget::ToolButtonWidget(const ActionsManager::ActionEntryDefinition &definition, QWidget *parent) : QToolButton(parent),
+ToolButtonWidget::ToolButtonWidget(const ToolBarsManager::ToolBarDefinition::Entry &definition, QWidget *parent) : QToolButton(parent),
+	m_parameters(definition.parameters),
 	m_isCustomized(false)
 {
 	setAutoRaise(true);
@@ -42,25 +45,37 @@ ToolButtonWidget::ToolButtonWidget(const ActionsManager::ActionEntryDefinition &
 
 	if (!definition.entries.isEmpty())
 	{
-		menu = new Menu(Menu::NoMenuRole, this);
+		menu = new Menu(Menu::UnknownMenu, this);
 
 		addMenu(menu, definition.entries);
 		setMenu(menu);
 	}
+	else if (definition.action == QLatin1String("OptionMenu") && definition.options.contains(QLatin1String("option")))
+	{
+		menu = new Menu(Menu::UnknownMenu, this);
+		menu->load(SettingsManager::getOptionIdentifier(definition.options[QLatin1String("option")].toString()));
+
+		setDefaultAction(menu->menuAction());
+	}
 	else if (definition.action.endsWith(QLatin1String("Menu")))
 	{
-		menu = new Menu(Menu::getRole(definition.action), this);
+		menu = new Menu(Menu::getMenuRoleIdentifier(definition.action), this);
 
 		setDefaultAction(menu->menuAction());
 	}
 
 	if (menu)
 	{
+		menu->setActionParameters(definition.parameters);
+		menu->setMenuOptions(definition.options);
+
 		setPopupMode(QToolButton::InstantPopup);
-		setText(definition.options.value(QLatin1String("text"), tr("Menu")).toString());
+		setText(getText());
+		setToolTip(getText());
+		setIcon(getIcon());
 	}
 
-	ToolBarWidget *toolBar(qobject_cast<ToolBarWidget*>(parent));
+	const ToolBarWidget *toolBar(qobject_cast<ToolBarWidget*>(parent));
 
 	if (toolBar)
 	{
@@ -68,9 +83,9 @@ ToolButtonWidget::ToolButtonWidget(const ActionsManager::ActionEntryDefinition &
 		setIconSize(toolBar->getIconSize());
 		setMaximumButtonSize(toolBar->getMaximumButtonSize());
 
-		connect(toolBar, SIGNAL(buttonStyleChanged(Qt::ToolButtonStyle)), this, SLOT(setButtonStyle(Qt::ToolButtonStyle)));
-		connect(toolBar, SIGNAL(iconSizeChanged(int)), this, SLOT(setIconSize(int)));
-		connect(toolBar, SIGNAL(maximumButtonSizeChanged(int)), this, SLOT(setMaximumButtonSize(int)));
+		connect(toolBar, &ToolBarWidget::buttonStyleChanged, this, &ToolButtonWidget::setButtonStyle);
+		connect(toolBar, &ToolBarWidget::iconSizeChanged, this, &ToolButtonWidget::setIconSize);
+		connect(toolBar, &ToolBarWidget::maximumButtonSizeChanged, this, &ToolButtonWidget::setMaximumButtonSize);
 	}
 }
 
@@ -94,34 +109,46 @@ void ToolButtonWidget::paintEvent(QPaintEvent *event)
 
 	initStyleOption(&option);
 
-	option.text = option.fontMetrics.elidedText(option.text, Qt::ElideRight, (option.rect.width() - (option.fontMetrics.width(QLatin1Char(' ')) * 2) - ((toolButtonStyle() == Qt::ToolButtonTextBesideIcon) ? iconSize().width() : 0)));
+	option.text = option.fontMetrics.elidedText(option.text, (isLeftToRight() ? Qt::ElideRight : Qt::ElideLeft), (option.rect.width() - (option.fontMetrics.width(QLatin1Char(' ')) * 2) - ((toolButtonStyle() == Qt::ToolButtonTextBesideIcon) ? iconSize().width() : 0)));
 
 	painter.drawComplexControl(QStyle::CC_ToolButton, option);
 }
 
-void ToolButtonWidget::addMenu(Menu *menu, const QList<ActionsManager::ActionEntryDefinition> &entries)
+void ToolButtonWidget::addMenu(Menu *menu, const QVector<ToolBarsManager::ToolBarDefinition::Entry> &entries)
 {
+	const ToolBarWidget *toolBar(qobject_cast<ToolBarWidget*>(parentWidget()));
+	ActionExecutor::Object executor(Application::getInstance(), Application::getInstance());
+
+	if (toolBar && toolBar->getMainWindow())
+	{
+		executor = ActionExecutor::Object(toolBar->getMainWindow(), toolBar->getMainWindow());
+	}
+
 	for (int i = 0; i < entries.count(); ++i)
 	{
-		if (entries.at(i).entries.isEmpty())
+		const ToolBarsManager::ToolBarDefinition::Entry entry(entries.at(i));
+
+		if (entry.entries.isEmpty())
 		{
-			if (entries.at(i).action.isEmpty() || entries.at(i).action == QLatin1String("separator"))
+			if (entry.action.isEmpty() || entry.action == QLatin1String("separator"))
 			{
 				menu->addSeparator();
 			}
 			else
 			{
-				menu->addAction(ActionsManager::getActionIdentifier(entries.at(i).action), true);
+				menu->addAction(new Action(ActionsManager::getActionIdentifier(entry.action), entry.parameters, entry.options, executor, menu));
 			}
 		}
 		else
 		{
 			Menu *subMenu(new Menu());
-			Action *subMenuAction(menu->addAction());
-			subMenuAction->setText(entries.at(i).options.value(QLatin1String("text"), tr("Menu")).toString());
+			QAction *subMenuAction(new QAction(menu));
+			subMenuAction->setText(entry.options.value(QLatin1String("text"), tr("Menu")).toString());
 			subMenuAction->setMenu(subMenu);
 
-			addMenu(subMenu, entries.at(i).entries);
+			menu->addAction(subMenuAction);
+
+			addMenu(subMenu, entry.entries);
 		}
 	}
 }
@@ -205,13 +232,6 @@ QString ToolButtonWidget::getText() const
 
 	if (defaultAction())
 	{
-		Action *action(qobject_cast<Action*>(defaultAction()));
-
-		if (action)
-		{
-			return action->getText();
-		}
-
 		return defaultAction()->text();
 	}
 
@@ -222,14 +242,14 @@ QIcon ToolButtonWidget::getIcon() const
 {
 	if (m_isCustomized && m_options.contains(QLatin1String("icon")))
 	{
-		const QString data(m_options[QLatin1String("icon")].toString());
+		const QVariant iconData(m_options[QLatin1String("icon")]);
 
-		if (data.startsWith(QLatin1String("data:image/")))
+		if (iconData.type() == QVariant::Icon)
 		{
-			return QIcon(QPixmap::fromImage(QImage::fromData(QByteArray::fromBase64(data.mid(data.indexOf(QLatin1String("base64,")) + 7).toUtf8()))));
+			return iconData.value<QIcon>();
 		}
 
-		return ThemesManager::getIcon(data);
+		return ThemesManager::createIcon(iconData.toString());
 	}
 
 	return (defaultAction() ? defaultAction()->icon() : icon());
@@ -240,6 +260,11 @@ QVariantMap ToolButtonWidget::getOptions() const
 	return m_options;
 }
 
+QVariantMap ToolButtonWidget::getParameters() const
+{
+	return m_parameters;
+}
+
 bool ToolButtonWidget::isCustomized() const
 {
 	return m_isCustomized;
@@ -247,9 +272,25 @@ bool ToolButtonWidget::isCustomized() const
 
 bool ToolButtonWidget::event(QEvent *event)
 {
-	if (event->type() == QEvent::ContextMenu && contextMenuPolicy() == Qt::NoContextMenu)
+	switch (event->type())
 	{
-		return false;
+		case QEvent::MouseButtonPress:
+		case QEvent::MouseButtonRelease:
+			if (!isEnabled())
+			{
+				return false;
+			}
+
+			break;
+		case QEvent::ContextMenu:
+			if (contextMenuPolicy() == Qt::NoContextMenu)
+			{
+				return false;
+			}
+
+			break;
+		default:
+			break;
 	}
 
 	return QToolButton::event(event);
